# -*- coding: utf-8 -*-

"""
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

    Development Team: Brain Storm Team
"""

# noinspection PyCompatibility
from argparse import ArgumentParser


class ArgumentParserError(Exception):

    """ArgumentParserError class"""
    pass


class ThrowingArgumentParser(ArgumentParser):

    """ThrowingArgumentParser class"""

    @classmethod
    def error(cls, message):
        """
        Error message
        :param message: message
        :raise ArgumentParserError
        :return: None
        """

        raise ArgumentParserError(message)


class OptionsError(Exception):

    """OptionsError class"""

    def __init__(self, message):
        """
        Error message
        :param message: message
        :return: None
        """

        super(OptionsError, self).__init__(message)


class FilterError(Exception):

    """FilterError class"""

    def __init__(self, message):
        """
         Error message
         :param message: message
         :return: None
         """

        super(FilterError, self).__init__(message)



# -*- coding: utf-8 -*-

"""
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

    Development Team: Brain Storm Team
"""

from src.core import filesystem, FileSystemError
from src.lib import tpl


class PluginProvider(object):
    """"PluginProvider class"""
    
    PLUGIN_NAME = 'PluginProvider'
    EXTENSION_SET = '.pp'
    
    def __init__(self, target, data):
        """
        PluginProvider constructor
        :param str target: target host
        :param dict data: result set
        """

        self._target = str(target)
        self._data = {}
        self.__set_data(data)

    def __set_data(self, data):
        """
        Set report data
        :param dict data:
        :return:
        """

        if False is isinstance(data, dict):
            raise TypeError("Report data has a wrong type")
        self._data = data

    def process(self):
        """
        Process data
        :return: mixed
        """

        pass
    
    @classmethod
    def record(cls, dirname, filename, resultset, separator=''):
        """
        Record data process
        :param str dirname: report directory
        :param str filename: report filename
        :param list resultset: report result
        :param str separator: result separator
        :raise Exception
        :return: None
        """
        
        try:
            filename = "".join((dirname, filesystem.sep, filename, cls.EXTENSION_SET))
            filename = filesystem.makefile(filename)
            filesystem.writelist(filename, resultset, separator)
            tpl.info(key='report', plugin=cls.PLUGIN_NAME, dest=filesystem.getabsname(filename))
        except FileSystemError as error:
            raise Exception(error)



# -*- coding: utf-8 -*-

"""
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

    Development Team: Brain Storm Team
"""

import platform
import shlex
import struct
import subprocess


class Terminal(object):

    """ Terminal class"""
    
    def get_ts(self):
        """
        Get width and height of console
        :return: tuple
        """
    
        current_os = platform.system()
        tuple_xy = (80, 25)  # default value
        if current_os == 'Windows':
            tuple_xy = self.__get_ts_windows()
            if tuple_xy is None:
                tuple_xy = self.__get_ts_tput()
        if current_os in ['Linux', 'Darwin'] or current_os.startswith('CYGWIN'):
            tuple_xy = self.__get_ts_unix()
        return tuple_xy
    
    @staticmethod
    def __get_ts_windows():
        """
        Get windows terminal size
        :return: tuple
        """

        # noinspection PyBroadException
        try:
            from ctypes import windll, create_string_buffer
            (sizex, sizey) = 25, 80  # default value
            h = windll.kernel32.GetStdHandle(-12)
            csbi = create_string_buffer(22)
            res = windll.kernel32.GetConsoleScreenBufferInfo(h, csbi)
            if res:
                (bufx, bufy, curx, cury, wattr,
                 left, top, right, bottom,
                 maxx, maxy) = struct.unpack("hhhhHhhhhhh", csbi.raw)
                sizex = right - left + 1
                sizey = bottom - top + 1
            return sizex, sizey
        except Exception:
            pass

    @staticmethod
    def __get_ts_unix():
        """
        Get unix terminal size
        :return tuple
        """
        (height, width) = 25, 80
        
        try:
            (height, width) = subprocess.check_output(['stty', 'size']).split()
        except (AttributeError, subprocess.CalledProcessError):
            subprocess.check_output = Terminal.__legacy_call
            (height, width) = subprocess.check_output(['stty', 'size']).split()
        finally:
            return width, height

    @staticmethod
    def __legacy_call(*popenargs, **kwargs):
        """
        Subprocess check output for legacy python version 2.6
        :param popenargs:
        :param kwargs:
        :return:
        """
        
        if 'stdout' in kwargs:
            raise ValueError('stdout argument not allowed, it will be overridden.')
        process = subprocess.Popen(stdout=subprocess.PIPE, *popenargs, **kwargs)
        output, unused_err = process.communicate()
        retcode = process.poll()
        if retcode:
            cmd = kwargs.get("args")
            if cmd is None:
                cmd = popenargs[0]
            raise subprocess.CalledProcessError(retcode, cmd)
        del unused_err
        return output
        
    @staticmethod
    def __get_ts_tput():
        """
        Get terminal height / width
        :return: tuple
        """
    
        try:
            cols = int(subprocess.check_call(shlex.split('tput cols')))
            rows = int(subprocess.check_call(shlex.split('tput lines')))
            return cols, rows
        except subprocess.CalledProcessError:
            pass



# -*- coding: utf-8 -*-

"""
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

    Development Team: Brain Storm Team
"""


class CookiesProvider(object):
    """ CookiesProvider class"""

    def __init__(self):
        """
        Init interface
        """

        self._cookies = None

    @property
    def _is_cookie_fetched(self):
        """
        Check if cookies have been fetched from response
        :return: bool
        """

        return False if None is self._cookies else True

    def _fetch_cookies(self, headers):
        """
        Fetch cookies from response
        :param dict headers:  header
        :return: None
        """

        if 'set-cookie' in headers:
            self._cookies = headers.get('set-cookie')

    def _push_cookies(self):
        """
        Push cookies to request
        :return: str cookies
        """

        return self._cookies.strip()



# -*- coding: utf-8 -*-

"""
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

    Development Team: Brain Storm Team
"""
from src.core import helper
from src.core import sys
from src.core.http.providers.debug import DebugProvider
# noinspection PyPep8Naming
from src.lib.tpl import Tpl as tpl


class Debug(DebugProvider):
    """Debug class"""

    # noinspection PyPep8Naming
    def __init__(self, Config):
        """
        Debug constructor
        :param Config: Config
        """

        self.__catched = False
        self.__clear = False
        self.__cfg = Config
        self.__level = self.__cfg.debug

        if 0 < self.__level:
            tpl.debug(key='debug', level=self.__cfg.debug, method=self.__cfg.method)

    @property
    def level(self):
        """
        Get debug level
        :return: int
        """

        return self.__level

    def debug_user_agents(self):
        """
        Debug info for user agent
        :return: bool
        """

        if 0 < self.__level:
            if True is self.__cfg.is_random_user_agent:
                tpl.debug(key='random_browser')
            else:
                tpl.debug(key='browser', browser=self.__cfg.user_agent)

        return True

    def debug_list(self, total_lines):
        """
        Debug scan list
        :param int total_lines:  lines
        :return: bool
        """

        if 0 < self.__level:
            if True is self.__cfg.is_random_list:
                tpl.debug(key='randomizing')
            if self.__cfg.DEFAULT_SCAN is self.__cfg.scan:
                if True is self.__cfg.is_extension_filter:
                    tpl.debug(key='ext_filter', total=total_lines, ext=', '.join(self.__cfg.extensions))
                elif True is self.__cfg.is_ignore_extension_filter:
                    tpl.debug(key='ext_ignore_filter', total=total_lines, ext=', '.join(self.__cfg.ignore_extensions))
                else:
                    tpl.debug(key='directories', total=total_lines)
            else:
                tpl.debug(key='subdomains', total=total_lines)
            tpl.debug(key='create_queue', threads=self.__cfg.threads)

        return True

    def debug_connection_pool(self, keymsg, pool, connection_type):
        """
        Debug connection pool message
        :param str keymsg: tpl key
        :param object pool: pool object
        :param object connection_type: type string
        :return: bool
        """

        tpl.debug(key=keymsg, type=connection_type)
        if pool:
            tpl.debug(str(pool), type=connection_type)

        return True

    def debug_proxy_pool(self):
        """
        Debug proxy pool message
        :return: bool
        """

        if True is self.__cfg.is_external_torlist:
            tpl.debug(key='proxy_pool_external_start')

        elif True is self.__cfg.is_standalone_proxy:
            tpl.debug(key='proxy_pool_standalone', server=self.__cfg.proxy)

        elif True is self.__cfg.is_internal_torlist:
            tpl.debug(key='proxy_pool_internal_start')

        return True

    def debug_request(self, request_header, url, method):
        """
        Debug request
        :param dict request_header: request header
        :param str url: request url
        :param str method: request method
        :return: bool
        """
        if type(request_header) is not dict:
            request_header = request_header.__dict__

        request_header.update({'Request URI': url})
        request_header.update({'Request Method': method})

        tpl.debug(key='request_header_dbg', dbg=helper.to_json(request_header))

        return True

    def debug_response(self, response_header):
        """
        Debug response
        :param dict response_header: response header
        :return: bool
        """

        tpl.debug(key='response_header_dbg', dbg=helper.to_json(response_header))

        return True

    def debug_request_uri(self, status, request_uri, **kwargs):
        """
        Debug request_uri
        :param str status: response status
        :param str request_uri: http request uri
        :param mixed kwargs:
        :return: bool
        """

        percentage = tpl.line(
            msg=helper.percent(kwargs.get('items_size', 0), kwargs.get('total_size', 1)),
            color='cyan')
        total_len = len(str(abs(kwargs.get('total_size', 1))))

        if self.__cfg.DEFAULT_SCAN is self.__cfg.scan:
            urlpath = helper.parse_url(request_uri).path
        else:
            urlpath = request_uri

        if status in ['success', 'file', 'indexof', 'certificate', 'auth']:
            request_uri = tpl.line(key=status, color='green', url=urlpath)
        elif status in ['bad', 'forbidden']:
            request_uri = tpl.line(key='forbidden', color='yellow', url=urlpath)
        elif status in ['redirect']:
            request_uri = tpl.line(key='redirect', color='blue', url=urlpath,
                                   rurl=kwargs.get('redirect_uri'))

        self.__clear = True if self.__catched else False

        if status in ['success', 'file', 'bad', 'forbidden', 'redirect', 'indexof', 'certificate', 'auth']:

            sys.writels("", flush=True)
            tpl.info(key='get_item',
                     clear=self.__clear,
                     percent=percentage,
                     current='{0:0{l}d}'.format(kwargs.get('items_size', 0), l=total_len),
                     total=kwargs.get('total_size', 1),
                     item=request_uri,
                     size=kwargs.get('content_size')
                     )
            self.__catched = True
        else:
            if self.__level != -1:
                tpl.line_log(key='get_item',
                             percent=percentage,
                             current='{0:0{l}d}'.format(kwargs.get('items_size', 0), l=total_len),
                             total=kwargs.get('total_size', 1),
                             item=request_uri,
                             size=kwargs.get('content_size'),
                             )
            self.__catched = False
            if kwargs.get('items_size', 0) is kwargs.get('total_size', 1):
                sys.writels("", flush=True)

        return True

    def debug_load_sniffer_plugin(self, description):
        """
        Debug load sniffers plugin
        :param str description: plugin description
        :return: bool
        """

        tpl.debug(key='load_sniffer_plugin', description=description)

        return True



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

"""
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

    Development Team: Brain Storm Team
"""

import sys
from .controller import Controller
from .exceptions import SrcError

for _ in ['urllib3', 'json2html', 'tabulate', 'importlib', 'packaging']:
    try:
        __import__(_)
    except ImportError as error:
        sys.exit("""\t\t[!] Several dependencies wasn't installed! Please run pip install -r requirements.txt. 
        Details : %s.""" % error)


def main():
    """
    This function loads the package and runs the Controller class.

    :except SrcError: Raises an exception if there is an error in the source code.
    :return: None
    """
    try:
        bootstrap = Controller()
        bootstrap.run()
    except SrcError:
        sys.exit()


if __name__ == "__main__":
    main()



# -*- coding: utf-8 -*-

"""
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

    Development Team: Brain Storm Team
"""

import os

from src.core import filesystem
from src.core import CoreConfig


def exception_log():
    """
    Get exception log path
    :return: string
    """

    exception_filelog = CoreConfig.get('data').get('exceptions_log')
    return filesystem.getabsname(exception_filelog)


class Config(object):

    """Config class"""

    logdir = os.path.dirname(exception_log())
    exceptions = {
        "version": 1,
        "disable_existing_loggers": False,
        "formatters": {
            "exception_format": {
                "format": "%(asctime)s [%(levelname)s] : %(message)s",
                "datefmt": "%Y-%m-%d %H:%M:%S"
            }
        },
        "handlers": {
            "exception_file_handler": {
                "level": "ERROR",
                "formatter": "exception_format",
                "class": "logging.handlers.RotatingFileHandler",
                "filename": exception_log(),
                "maxBytes": 10485760,
                "backupCount": 10
            }
        },
        "loggers": {
            "exceptions": {
                "handlers": ["exception_file_handler"]
            }
        }
    }



# -*- coding: utf-8 -*-

"""
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

    Development Team: Brain Storm Team
"""

from difflib import SequenceMatcher
from .provider import ResponsePluginProvider


class CollationResponsePlugin(ResponsePluginProvider):
    """ CollationResponsePlugin class"""

    DESCRIPTION = 'Collation (detect and ignore false positive success pages)'
    RESPONSE_INDEX = 'failed'
    DEFAULT_STATUSES = [100, 101, 200, 201, 202, 203, 204, 205, 206, 207, 208]
    MIN_RATIO_INDEX = 0.98
    MIN_CONTENT_LENGTH = 100

    def __init__(self, void):
        """
        ResponsePluginProvider constructor
        """

        ResponsePluginProvider.__init__(self)
        self.strong_false_item = {}
        self.previous_item = {}
        self.current_item = {}

    def process(self, response):
        """
        Process data
        :return: str
        """

        if response.status in self.DEFAULT_STATUSES:
            super().process(response)
            length = self.__get_content_length()
            if self.MIN_CONTENT_LENGTH < length:
                # the page is allowed for comparison

                if not self.previous_item:
                    # 1st match. Push items for next compare step
                    self.previous_item.update({'length': length, 'text': self._body})
                    return None
                else:
                    if length == self.previous_item.get('length') and self.MIN_CONTENT_LENGTH < length:
                        # identical, seems to drop failed for success
                        return self.RESPONSE_INDEX
                    else:
                        matcher = SequenceMatcher(a=self.previous_item['text'], b=self._body)
                        matcher.get_matching_blocks()

                        if 'length' in self.current_item:
                            next_matcher = SequenceMatcher(a=self.current_item['text'], b=self._body)
                            if next_matcher.ratio() == matcher.ratio():
                                return self.RESPONSE_INDEX
                        if self.MIN_RATIO_INDEX < matcher.ratio():
                            return self.RESPONSE_INDEX
                        else:
                            self.current_item.update({'length': length, 'text': self._body})

                    if self.MIN_CONTENT_LENGTH < length:
                        self.previous_item.update({'length': length, 'text': self._body})
        return None

    def __get_content_length(self):
        """
        Get content length
        :return: int
        """

        length = 0
        if 'Content-Length' in self._headers:
            if 0 < int(self._headers['Content-Length']):
                length = self._headers['Content-Length']
        else:
            length = len(self._body)
        return int(length)



# -*- coding: utf-8 -*-

"""
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

    Development Team: Brain Storm Team
"""

from __future__ import absolute_import
import unittest
from ddt import ddt, data
from src.core.options.filter import Filter

@ddt
class TestFilter(unittest.TestCase):
    """TestFilter class"""
    
    @data(
            {'host': 'example.com', 'port': 80, 'debug': 1},
            {'host': 'https://example.com', 'port': 223, 'debug': 1},
            {'host': 'example.com','debug': 1, 'scan': 'directories'},
            {'host': 'example.com','debug': 1, 'scan': 'subdomains'}
    )
    def test_filter(self, args):
        """ Filter.filter() test """
        
        flt = Filter.filter(args)
        self.assertIs(type(flt), dict)

      
        
if __name__ == "__main__":
    unittest.main()



# -*- coding: utf-8 -*-

"""
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

    Development Team: Brain Storm Team
"""

import random


class AcceptHeaderProvider(object):
    """ AcceptHeaderProvider class"""

    def __init__(self):
        """
        Init interface
        """

        self.__accept = '*/*'

        self.__accept_encoding = 'identity'

        self.__accept_language = (
            'en-US,en;q=0.5,ru-RU,ru;q=0.8',
            'ru-RU,ru;q=0.8,en-US;q=0.6,en;q=0.4,uk;q=0.2,es;q=0.2,pl;q=0.2',
            'en,en-gb;q=0.8, en;q=0.7'
        )

    @property
    def _accept(self):
        """
        Get 'Accept' Header
        :return: str
        """

        return self.__accept

    @property
    def _accept_encoding(self):
        """
        Get 'Accept-Encoding' Header
        :return: str
        """

        return self.__accept_encoding

    @property
    def _accept_language(self):
        """
        Get 'Accept-Language' Header
        :return: str
        """

        index = random.randrange(0, len(self.__accept_language))
        accept = self.__accept_language[index]

        return accept



# -*- coding: utf-8 -*-

"""
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

    Development Team: Brain Storm Team
"""


class Config(object):

    """Config class"""

    templates = {
        'unsupported': 'Your Python version v{actual} is not supported by this application. Please consider {expected}',
        'abort': 'Session canceled',
        'upd_win_stat': 'Win OS does not support --update inside.\nPlease run your GIT binary to fetch data manualy',
        'use_reports': 'Use --report param to store your scan results',
        'load_wizard': 'Load wizard options from : {config}',
        'report': '{plugin} : {dest}',
        'logged': 'Scanned host has been reported. Press ENTER to rescan or CTRL+C to exit: ',
        'checking_connect': 'Wait, please, checking connect to -> {host}:{port} ...',
        'slow_connection': 'Too slow connection. Please decrease the num of threads and increase the request timeout',
        'online': 'Server {host}:{port} ({ip}) is online!',
        'create_queue_progress': 'Create queue {bar}',
        'scanning': 'Scanning {host} ...',
        'debug': 'Starting debug level {level}. Using scan method: {method} ...',
        'randomizing': 'Randomizing scan list ...',
        'browser': 'Fetching user-agent: {browser}',
        'directories': 'Read {total} directories list by line',
        'ext_filter': 'Read {total} directories list required only: {ext}',
        'ext_ignore_filter': 'Read {total} directories exclude only: {ext}',
        'create_queue': 'Creating queue with {threads} thread(s)...',
        'subdomains': 'Read {total} subdomains list by line...',
        'random_browser': 'Fetching random user-agent per request...',
        'total_time_lvl3': 'Total time running: {time}',
        'thread_limit': 'Threads has been reduced to {max} (max) instead of {threads}',
        'stop_threads': 'Stopping threads ({threads})...',
        'option_prompt': 'Press "[C]ontinue" to resume or "[E]xit" to abort session: ',
        'resume_threads': 'Resuming scan...',
        'get_item': '{percent} [{current}/{total}] - {size} - {item}',
        'ignored_item': 'skip [{current}/{total}] - Ignored {item}',
        'max_retry_error': 'skip. Max retries exceeded: {url}',
        'addtopool': 'Adding {total} lines to queue...',
        'http_pool_start': 'Using HTTP {type} connection',
        'https_pool_start': 'Using SSL {type} connection',
        'proxy_pool_standalone': 'Using custom proxy server: {server}',
        'proxy_pool_internal_start': 'Fetching internal proxy list...',
        'proxy_pool_external_start': 'Fetching external proxy list...',
        'request_header_dbg': 'Request header:\n{dbg}',
        'response_header_dbg': 'Response header:\n{dbg}',
        'load_sniffer_plugin': 'Load sniffer: {description}',
        'ignored_path': 'Ignored. The path {path} in ignore list',
        'proxy_max_retry_error': 'Skipped. Proxy {proxy} Max retries exceeded: {url}',
        'host_changed_error': 'Block external redirect -> {details}',
        'read_timeout_error': 'Read timeout error! {url}. Increase using --timeout option',
        'connection_timeout_error': 'Connection timeout error! {url}. Increase using --timeout option',
        'certificate': 'Cert required {url}',
        'success': 'OK {url}',
        'file': 'File {url}',
        'indexof': 'Index {url}',
        'forbidden': 'Denied {url}',
        'auth': 'Auth {url}',
        'redirect': 'R {url} -> {rurl}'
    }



# -*- coding: utf-8 -*-

"""
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

    Development Team: Brain Storm Team
"""

from urllib3 import HTTPHeaderDict

from .accept import AcceptHeaderProvider
from .cache import CacheControlProvider


class HeaderProvider(AcceptHeaderProvider, CacheControlProvider):
    """ HeaderProvider class"""

    def __init__(self, config):
        """
        Init interface.
        Accept external params
        :param src.lib.browser.config.Config config: browser configurations
        """

        self.__headers = HTTPHeaderDict()

        self.__cfg = config

        AcceptHeaderProvider.__init__(self)
        CacheControlProvider.__init__(self)

    def add_header(self, key, value):
        """
        Add custom header

        :param str key: header name
        :param str value: header value
        :return: HeaderProvider
        """

        self.__headers.update({key.strip(): value.strip()})
        return self

    @property
    def _headers(self):
        """
        Get Headers
        :return: dict headers
        """

        origin = ''.join([self.__cfg.scheme, self.__cfg.host])
        referer = ''.join([self.__cfg.scheme, self.__cfg.host]) + ':' + str(self.__cfg.port)
        self.add_header('Accept', self._accept) \
            .add_header('Accept-Encoding', self._accept_encoding) \
            .add_header('Accept-Language', self._accept_language) \
            .add_header('Origin', origin) \
            .add_header('Referer', referer) \
            .add_header('Cache-Control', self._cache_control) \
            .add_header('Upgrade-Insecure-Requests', '1') \
            .add_header('Pragma', 'no-cache')
        return self.__headers



# -*- coding: utf-8 -*-

"""
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

    Development Team: Brain Storm Team
"""

import unittest
from src.lib.browser.debug import Debug
from src.lib.browser.config import Config
from mock import patch
from src.core.logger.logger import Logger
from io import StringIO
from ddt import ddt, data


@ddt
class TestBrowserDebug(unittest.TestCase):
    """TestBrowserDebug class"""

    CONFIG = Config({
        'debug' : 1,
        'method' : 'HEAD',
        'indexof' : None,
        'random_agent': False,
        'random_list' : True,
        'threads' : 1
    })

    def setUp(self):
        with patch('sys.stdout', new=StringIO()):
            self.debug = Debug(self.CONFIG)

    def tearDown(self):
        StringIO().flush()
        logger = Logger.log()
        for handler in logger.handlers:
            logger.removeHandler(handler)
        del self.debug

    def test_level(self):
        """ Debug.level test """

        self.assertIs(type(self.debug.level), int)
        self.assertTrue(0 < self.debug.level)

    @data('success', 'file', 'indexof', 'bad','forbidden', 'redirect', 'certificat', 'auth')
    def test_debug_request_uri(self, status):
        """ Debug.debug_request_uri() test """

        setattr(self.debug, '_Reader__cfg.DEFAULT_SCAN', 'directories')
        if 'forbidden' in status:
            setattr(self.debug, '_Reader__cfg.scan', 'directories')
        else:
            setattr(self.debug, '_Reader__cfg.scan', 'subdomains')

        with patch('sys.stdout', new=StringIO()):
            self.assertTrue(self.debug.debug_request_uri(status,'http://test.local/data/'))

    def test_debug_request(self):
        """ Debug.debug_request() test """

        self.assertTrue(self.debug.debug_request({},'http://test.local/data/', 'HEAD'))

    def test_debug_response(self):
        """ Debug.debug_response() test """

        self.assertTrue(self.debug.debug_response(response_header={}))

    def test_debug_list(self):
        """ Debug.debug_list() test """

        self.assertTrue(self.debug.debug_list(1))

    def test_debug_connection_pool(self):
        """ Debug.debug_connection_pool() test """

        self.assertTrue(self.debug.debug_connection_pool('http_pool_start', '', ''))

    def test_debug_proxy_pool(self):
        """ Debug.debug_proxy_pool() test """

        self.assertTrue(self.debug.debug_proxy_pool())

    def test_debug_user_agents(self):
        """ Debug.debug_user_agents() test """
        self.assertTrue(self.debug.debug_user_agents())


if __name__ == "__main__":
    unittest.main()


# -*- coding: utf-8 -*-

"""
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

    Development Team: Brain Storm Team
"""

from src.core import exception


class PackageError(Exception):

    """PackageError class"""

    def __init__(self, error):
        """
        PackageError class constructor
        :param str error: error message
        """

        class_name = type(error).__name__

        if self.__class__.__name__ is not class_name:
            exception.log(class_name=class_name, message=error)

        super(PackageError, self).__init__("{0}: {1}".format(str(class_name), str(error)))



# -*- coding: utf-8 -*-

"""
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

    Development Team: Brain Storm Team
"""


class DebugProvider(object):
    """ DebugProvider class"""

    @property
    def level(self):
        """
        Get debug level
        :return: int
        """

        return None

    def debug_user_agents(self):
        """
        Debug info for user agent
        :return: bool
        """

        pass

    def debug_connection_pool(self, keymsg, pool, connection_type):
        """
        Debug connection pool message
        :param str keymsg: tpl key
        :param object pool: pool object
        :param object connection_type: type string
        :return: bool
        """

        pass

    def debug_proxy_pool(self):
        """
        Debug proxy pool message
        :return: bool
        """

        pass

    def debug_list(self, total_lines):
        """
        Debug scan list
        :param int total_lines: total list lines
        :return: bool
        """

        pass

    def debug_request(self, request_header, url, method):
        """
        Debug request
        :param dict request_header: request header
        :param str url: request url
        :param str method: request method
        :return: bool
        """

        pass

    def debug_response(self, response_header):
        """
        Debug response
        :param dict response_header: response header
        :return: bool
        """

        pass

    def debug_request_uri(self, status, request_uri, **kwargs):
        """
        Debug request_uri
        :param int status: response status
        :param str request_uri: request urli
        :return: bool
        """

        pass

    def debug_load_sniffer_plugin(self, description):
        """
        Debug load sniffers plugin
        :param str description: plugin description
        :return: bool
        """

        pass



# -*- coding: utf-8 -*-

"""
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

    Development Team: Brain Storm Team
"""

import random


class UserAgentHeaderProvider(object):
    """ UserAgentHeaderProvider class"""

    def __init__(self, config, agent_list=()):
        """
        Init interface
        Parameters:
        agent_list (tuple, optional): A tuple containing the initial list of agents. Defaults to an empty tuple.
        """

        self.__cfg = config
        self.__agent_list = agent_list

    @property
    def _user_agent(self):
        """
        Get 'User-Agent' Header
        :return: str
        """

        if True is self.__cfg.is_random_user_agent:
            index = random.randrange(0, len(self.__agent_list))
            user_agent = self.__agent_list[index].strip()
        else:
            user_agent = self.__cfg.user_agent
        return user_agent



# -*- coding: utf-8 -*-

"""
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

    Development Team: Brain Storm Team
"""

import datetime
import functools
import time


def execution_time(func=None, log=None):
    """
    Time execution decorator @execution_time(log=tpl)
    :param funct func: wrapped function
    :param funct log: logger
    :return: func
    """

    if not func:
        # noinspection PyArgumentList
        return functools.partial(execution_time, log=log)

    @functools.wraps(func)
    def function_timer(*args, **kwargs):
        """
        Function timer
        :param args:  arguments
        :param kwargs: key arguments
        :return: func
        """

        start = time.time()
        result = func(*args, **kwargs)
        end = time.time()
        timeless = "{0}".format(datetime.timedelta(seconds=(end - start)))
        log.debug(key='total_time_lvl3', time=timeless)
        return result

    return function_timer



# -*- coding: utf-8 -*-

"""
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

    Development Team: Brain Storm Team
"""

import threading
import time
from queue import Empty as QueueEmptyError
from threading import BoundedSemaphore, Event

from src.core import process
# noinspection PyPep8Naming
from src.lib.tpl import Tpl as tpl


class Worker(threading.Thread):

    """Worker class"""

    def __init__(self, queue, num_threads, timeout=0):
        """
        Init thread worker
        :param Queue.Queue queue: simple queue object
        :param int num_threads: threads numbers
        :param int timeout: delay timeout
        """

        super(Worker, self).__init__()
        self.__semaphore = BoundedSemaphore(num_threads)
        self.__event = Event()
        self.__event.set()
        self.__empty = False
        self.__running = True
        self.__queue = queue
        self.__timeout = timeout
        self.counter = 0

    def pause(self):
        """
        Pause current worker
        :return: None
        """

        self.__running = False
        self.__event.clear()

    def resume(self):
        """
        Resume current worker
        :return: None
        """

        self.__running = True
        self.__event.set()

    def run(self):
        """
        Run current worker
        :return: None
        """

        try:

            self.__event.wait()

            while self.__running:

                if 0 < self.__timeout:
                    time.sleep(self.__timeout)

                try:
                    self.__process()
                except QueueEmptyError:
                    self.__empty = True

                finally:
                    if not self.__event.is_set():
                        self.__semaphore.release()
                        self.__event.wait()
        except Exception as error:
            self.terminate(str(error))

    def __process(self):
        """
        Task process
        :return: None
        """

        func, args, kargs = self.__queue.get(block=True)
        self.counter += 1
        func(*args, **kargs)
        self.__queue.task_done()

    @classmethod
    def terminate(cls, msg):
        """
        Terminate thread
        :param str msg: output message
        :return: None
        """

        tpl.error(msg)
        process.kill()



# -*- coding: utf-8 -*-

"""
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

    Development Team: Brain Storm Team
"""

import logging
import logging.config
import sys
import time
from inspect import currentframe
from .rainbow import RainbowLoggingHandler


class Logger(object):

    """ Logger class"""

    _record = None

    _levels = {'error': 40, 'warning': 30, 'info': 20, 'debug': 10}

    @staticmethod
    def inline(msg='', status='info'):
        """
        Formatted log message for inline console stdout
        :param str msg: formatted message
        :param str status: status name
        :return: str
        """

        if None is Logger._record:
            Logger._record = type('record', (object,),
                                  dict(exc_info=False,
                                       stack_info=False,
                                       exc_text=False,
                                       name='',
                                       levelno=Logger._levels.get(status),
                                       funcName=status,
                                       lineno=currentframe().f_back.f_lineno)
                                  )

        Logger._record.created = time.time()

        # noinspection PyPep8Naming
        def getMessage(__class__):
            """
            Emulate message

            :param __class__: current class
            :return: str
            """

            del __class__

            return msg

        setattr(Logger._record, 'getMessage', classmethod(getMessage))
        message = RainbowLoggingHandler().colorize(Logger._record)
        return message

    @staticmethod
    def log(name=__name__):
        """
        Log handler
        :param str name: log name
        :return: logging
        """

        logger = logging.getLogger(name)

        if not len(logger.handlers):
            logger.setLevel(logging.ERROR)
            logger.setLevel(logging.INFO)
            logger.setLevel(logging.WARNING)
            logger.setLevel(logging.DEBUG)
            handler = RainbowLoggingHandler(sys.stdout)
            logger.addHandler(handler)

        return logger



# -*- coding: utf-8 -*-

"""
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

    Development Team: Brain Storm Team
"""

from .cookies import CookiesProvider
from .header import HeaderProvider
from .user_agent import UserAgentHeaderProvider
from .connection import ConnectionHeaderProvider


class RequestProvider(CookiesProvider, HeaderProvider, UserAgentHeaderProvider, ConnectionHeaderProvider):
    """ RequestProvider class"""

    _HTTP_DBG_LEVEL = 2

    def __init__(self, config, agent_list):
        """
        Init interface
        :param src.lib.browser.config.Config config: configurations
        :param dict agent_list: list of user agents
        """

        HeaderProvider.__init__(self, config)
        ConnectionHeaderProvider.__init__(self, config)
        UserAgentHeaderProvider.__init__(self, config, agent_list)
        CookiesProvider.__init__(self)

    def request(self, url):
        """
        Client request
        :param str url: request uri
        :return: None
        """

        pass

    def cookies_middleware(self, is_accept, response):
        """
        Route fetched cookies from first response to the next requests
        :param is_accept: Is cookies was accepted
        :param urllib3.response.BaseHTTPResponse response: Http response
        :return: None
        """

        if True is is_accept and hasattr(response, 'headers'):
            self._fetch_cookies(response.headers)
            if True is self._is_cookie_fetched:
                self.add_header('Cookie', self._push_cookies())



# -*- coding: utf-8 -*-

"""
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

    Development Team: Brain Storm Team
"""

import socket
from .exceptions import SocketError


class Socket(object):

    """Socket class"""

    @staticmethod
    def ping(host, port, timeout=10):
        """
        Ping remote host
        :param str host: target  host
        :param int port: target port
        :param int timeout: connection timeout
        :raise SocketError
        :return: None
        """

        sock = socket.socket()

        try:

            sock.settimeout(timeout)
            sock.connect((host, port))

        except (socket.gaierror, socket.error, socket.timeout, SocketError) as error:
            raise SocketError(error)
        finally:
            sock.close()

    @staticmethod
    def get_ip_address(host):
        """
        Get remote ip address
        :param str host: target host
        :raise SocketError
        :return: str
        """

        try:
            ip_address = socket.gethostbyname(host)
            return ip_address
        except socket.gaierror as error:
            raise SocketError(str(error))

    @staticmethod
    def get_ips_addresses(host):
        """
        Get remote ip addresses
        :param str host: target host
        :return: list
        """

        try:
            _, _, ips_list = socket.gethostbyname_ex(host)
            if not ips_list:
                ips = ''
            else:
                ips = '['+', '.join(ips_list) +']'
        except socket.gaierror:
            ips = ''
        return ips



# -*- coding: utf-8 -*-

"""
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

    Development Team: Brain Storm Team
"""

from src.core import helper


class ResponsePluginProvider(object):
    """"ResponsePluginProvider class"""

    def __init__(self):
        """
        PluginProvider constructor
        """
        self._status = 0
        self._headers = {}
        self._body = ''

    def __set_body(self, body):
        """
        Set response data
        :param str body: response data
        :return: None
        """

        if False is isinstance(body, str):
            self._body = helper.decode(body)

    def process(self, response):
        """
        Process data from the given response object.
        :param urllib3.response.HTTPResponse response: The response object containing the data to be processed.
        :return: A string representing the processed data.
        """

        self._status = int(float(response.status))
        self._headers = response.headers
        self.__set_body(response.data)

        pass



# -*- coding: utf-8 -*-

"""
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

    Development Team: Brain Storm Team
"""

# noinspection PyPep8Naming
from src.lib.tpl import Tpl as tpl


class Filter(object):

    """Filter class"""

    # noinspection PyPep8Naming
    def __init__(self, Config, total_lines):
        """
        Filter constructor
        :param Config: Config
        :param int total_lines: num lines in list
        """

        if Config.threads > Config.DEFAULT_MAX_THREADS or Config.threads > total_lines:

            max_threads = total_lines if Config.DEFAULT_MAX_THREADS > total_lines else Config.DEFAULT_MAX_THREADS
            tpl.warning(key='thread_limit', threads=Config.threads, max=max_threads)
            Config.set_threads(max_threads)



# -*- coding: utf-8 -*-

"""
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

    Development Team: Brain Storm Team
"""

import threading
import time
from queue import Queue

# noinspection PyPep8Naming
from src.lib.tpl import Tpl as tpl
from .worker import Worker


class ThreadPool(object):

    """ThreadPool class"""

    def __init__(self, num_threads, total_items, timeout):
        """
        Initialize thread pool
        :param int num_threads: active workers
        :param int total_items: total items
        :param int timeout: delay between threads
        """

        self.__queue = Queue(num_threads)
        self.__workers = []
        self.total_items_size = total_items
        self.is_started = True

        for _ in range(num_threads):

            worker = Worker(self.__queue, num_threads, timeout)
            if False is worker.is_alive():
                worker.daemon = True
                worker.start()
                self.__workers.append(worker)

    @property
    def size(self):
        """
        Get pool size
        :return: int
        """

        return self.__queue.qsize()

    @property
    def workers_size(self):
        """
        Get pool workers (threads)
        :return: int
        """

        return self.__workers.__len__()

    @property
    def items_size(self):
        """
        Get pool items size
        :return: int
        """

        counter = 0
        for worker in self.__workers:
            counter += worker.counter
        return counter

    def add(self, func, *args, **kargs):
        """
        Add a task to the queue
        :param func func: callback function
        :param dict args: arguments
        :param kargs: keys arguments
        :return: None
        """

        try:
            if True is self.is_started:
                if self.items_size < self.total_items_size:
                    self.__queue.put((func, args, kargs))
        except (SystemExit, KeyboardInterrupt):
            time.sleep(2)
            self.pause()

    def join(self):
        """
        Join queue
        :return: None
        """

        self.__queue.join()

    def pause(self):
        """
        ThreadPool pause
        :raise KeyboardInterrupt
        :return: None
        """

        self.is_started = False
        tpl.info(key='stop_threads', threads=len(self.__workers))

        try:
            while 0 < threading.active_count():
                for worker in threading.enumerate():
                    if threading.current_thread().__class__.__name__ != '_MainThread':
                        worker.pause()
                time.sleep(2)

                char = tpl.prompt(key='option_prompt')
                if char.lower() == 'e':
                    raise KeyboardInterrupt
                elif char.lower() == 'c':
                    self.resume()
                    break
                else:
                    continue

        except (SystemExit, KeyboardInterrupt):
            raise KeyboardInterrupt

    def resume(self):
        """
        Resume threadpool
        :return: None
        """

        if False is self.is_started:
            tpl.info(key='resume_threads')
            for worker in self.__workers:
                worker.resume()
            self.is_started = True



# -*- coding: utf-8 -*-

"""
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

    Development Team: Brain Storm Team
"""

import unittest
from mock import patch
from src.lib.browser.threadpool import ThreadPool
from src.core.logger.logger import Logger


class TestBrowserThreadPool(unittest.TestCase):
    """TestBrowserThreadPool class"""
    
    THREADS = 2
    
    def __test_function(self,arg):
        pass
    
    def setUp(self):
        self._pool = ThreadPool(num_threads=self.THREADS, total_items=10, timeout=0)
    
    def tearDown(self):
        logger = Logger.log()
        for handler in logger.handlers:
            logger.removeHandler(handler)
        self._pool.join()

    def test_size(self):
        """ ThreadPool.size test """
        
        self.assertIs(type(self._pool.size), int)
        self.assertEqual(self._pool.size, 0)

    def test_worker_size(self):
        """ ThreadPool.worker_size test """
    
        self.assertIs(type(self._pool.workers_size), int)
        self.assertEqual(self._pool.workers_size, self.THREADS)

    def test_items_size(self):
        """ ThreadPool.items_size test """
    
        self.assertIs(type(self._pool.items_size), int)
        self.assertEqual(self._pool.items_size, 0)

    def test_add(self):
        """ ThreadPool.add() test """
    
        self.assertIs(self._pool.add(self.__test_function, 1), None)
    
    def pause(self):
        ans = self._pool.pause()
        return ans
    
    def test_pause(self):
        """ ThreadPool.pause() test """
        
        with self.assertRaises(KeyboardInterrupt) as context:
            with patch('builtins.input', return_value='e') as _raw_input:
                self.assertEqual(self.pause(), 'e')
                _raw_input.assert_called_once_with('e')
            self.assertTrue(KeyboardInterrupt == context.expected)

    def test_resume(self):
        """ ThreadPool.resume() test """
        
        self._pool.is_started = False
        self.assertIs(self._pool.resume(), None)


if __name__ == "__main__":
    unittest.main()


# -*- coding: utf-8 -*-

"""
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

    Development Team: Brain Storm Team
"""

from .provider import ResponsePluginProvider
from src.core.filesystem import FileSystem
from src.core.helper import Helper


class SkipSizesResponsePlugin(ResponsePluginProvider):
    """ SkipSizesResponsePlugin class"""

    DESCRIPTION = 'SkipSizesStatuses (skip target sizes of page: {} kbs for 200 OK redirects)'
    RESPONSE_INDEX = 'skip'
    DEFAULT_STATUSES = [200]
    SIZE_VALUES = None

    def __init__(self, values):
        """
        ResponsePluginProvider constructor
        """
        if values is not None:
            self.SIZE_VALUES = Helper.to_list(values, ':')
            size_list = list(map(lambda x: str(x)+'KB', self.SIZE_VALUES))
            size_string = ','.join(self.SIZE_VALUES)
            self.DESCRIPTION = self.DESCRIPTION.format(size_string)
            self.SIZE_VALUES = size_list
        ResponsePluginProvider.__init__(self)

    def process(self, response):
        """
        Process the given response data.

        :param response: The response object to be processed.
        :type response: object
        :return: The response index if the data meets the specified conditions, otherwise None.
        :rtype: str or None
        """
        if hasattr(response, 'status') and response.status in self.DEFAULT_STATUSES:
            super().process(response)
            if 'Content-Length' in self._headers:
                for size in self.SIZE_VALUES:
                    if size == FileSystem.human_size(int(self._headers['Content-Length']), 0):
                        return self.RESPONSE_INDEX
            else:
                for size in self.SIZE_VALUES:
                    if size == FileSystem.human_size(len(self._body), 0):
                        return self.RESPONSE_INDEX
        return None



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

""" OWASP OpenDoor launcher

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

    Development Team: Brain Storm Team
"""

import sys

for _ in ['urllib3', 'json2html', 'tabulate', 'importlib', 'packaging']:
    try:
        __import__(_)
    except ImportError as error:
        sys.exit("""\t\t[!] Several dependencies wasn't installed! Please run pip3 install -r requirements.txt. 
        Details : %s.""" % error)

if __name__ == "__main__":

    from src import Controller, SrcError
    import sys

    try:
        bootstrap = Controller()
        bootstrap.run()
    except SrcError:
        sys.exit()



# -*- coding: utf-8 -*-

"""
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

    Development Team: Brain Storm Team
"""

from src.core import FileSystemError
from src.core import CoreSystemError
from src.core import CoreConfig
from src.core import filesystem
from src.core import process
from src.core import sys
from src.core import helper
from .config import Config
from .exceptions import ReaderError


class Reader(object):

    """Reader class"""

    def __init__(self, browser_config):
        """
        Reader constructor
        :param dict browser_config:
        """

        self.__config = CoreConfig.get('data')
        self.__browser_config = browser_config
        self.__useragents = []
        self.__proxies = []
        self.__ignored = []
        self.__counter = 0

    def get_user_agents(self):
        """
        Get user agents from user-agents list
        :raise ReaderError
        :return: list
        """

        try:
            if not len(self.__useragents):
                self.__useragents = filesystem.read(self.__config.get('useragents'))
            return self.__useragents

        except (TypeError, FileSystemError) as error:
            raise ReaderError(error)

    def get_ignored_list(self):
        """
        Get ignored dir list
        :raise ReaderError
        :return: list
        """

        try:

            if not len(self.__ignored):
                ignored = filesystem.read(self.__config.get('ignored'))
                for item in ignored:
                    item = item.replace("\n", "")
                    if "/" == item[0]:
                        item = item.strip('/')
                    self.__ignored.append(item)

            return self.__ignored

        except (TypeError, FileSystemError) as error:
            raise ReaderError(error)

    def get_proxies(self):
        """
        Get a proxy list
        :raise ReaderError
        :return: list
        """

        try:
            if False is self.__browser_config.get('is_standalone_proxy'):

                if True is self.__browser_config.get('is_external_torlist'):
                    self.__proxies = filesystem.read(self.__browser_config.get('torlist'))
                else:

                    if not len(self.__proxies):
                        self.__proxies = filesystem.read(self.__config.get('proxies'))
                return self.__proxies
            else:
                return []

        except (TypeError, FileSystemError) as error:
            raise ReaderError(error)

    def get_lines(self, params, loader):
        """
        Read lines from large file
        :param dict params: input params
        :param funct loader:  callback function
        :raise ReaderError
        :return: None
        """

        try:
            if True is self.__browser_config.get('use_random'):
                # use randomizing list.dat
                dirlist = self.__config.get('tmplist')
            elif True is self.__browser_config.get('use_extensions')\
                    and 'directories' == self.__browser_config.get('list'):
                dirlist = self.__config.get('extensionlist')
            elif True is self.__browser_config.get('use_ignore_extensions')\
                    and 'directories' == self.__browser_config.get('list'):
                dirlist = self.__config.get('ignore_extensionlist')
            else:
                if True is self.__browser_config.get('is_external_wordlist'):
                    dirlist = self.__browser_config.get('wordlist')
                else:
                    dirlist = self.__config.get(self.__browser_config.get('list'))
            filesystem.readline(dirlist, handler=getattr(self, '_{0}__line'.format(self.__browser_config.get('list'))),
                                handler_params=params, loader=loader)
        except (TypeError, FileSystemError) as error:
            raise ReaderError(error)

    @classmethod
    def _subdomains__line(cls, line, params):
        """
        Read lines from subdomains file
        :param str line: single line
        :param dict params: input params
        :return: str
        """

        line = helper.filter_domain_string(line)

        host = params.get('host')
        port = params.get('port')

        if 'www.' in host:
            host = host.replace("www.", "")

        if port is Config.ssl_port or port is Config.http_port:
            port = ''
        else:
            port = ':{0}'.format(port)

        line = "{scheme}{sub}.{host}{port}".format(scheme=params.get('scheme'),
                                                   host=host,
                                                   port=port,
                                                   sub=line)

        return line

    def _directories__line(self, line, params):
        """
        Read lines from directories file
        :param str line: single line
        :param dict params: input params
        :return: str
        """

        line = helper.filter_directory_string(line)

        if 'prefix' in self.__browser_config and 0 < len(self.__browser_config.get('prefix')):
            line = self.__browser_config.get('prefix') + line
        port = params.get('port')

        if port is Config.ssl_port or port is Config.http_port:
            port = ''
        else:
            port = ':{0}'.format(port)
        line = "{scheme}{host}{port}/{uri}".format(scheme=params.get('scheme'), host=params.get('host'), port=port,
                                                   uri=line, )

        return line

    def randomize_list(self, target, output):
        """
        Randomize scan list
        :param str target: target list
        :param str output: output list
        :raise ReaderError
        :return: None
        """

        try:

            target_file = self.__config.get(target)
            tmp_file = self.__config.get(output)
            output_file = filesystem.makefile(tmp_file)

            if False is sys().is_windows:
                process.execute('shuf {target} -o {output}'.format(target=target_file, output=output_file))
            else:
                filesystem.shuffle(target=target_file, output=output_file, total=self.total_lines)
        except (CoreSystemError, FileSystemError) as error:
            raise ReaderError(error)

    def filter_by_extension(self, target, output, extensions):
        """
        Filter list by multiple extensions

        :param str target: target list
        :param str output: output list
        :param list extensions: filtered extensions
        :return: None
        """

        try:

            target_file = self.__config.get(target)
            output_file = self.__config.get(output)

            dirlist = filesystem.read(target_file)
            dirlist = [i.strip() for i in dirlist]
            pattern = '.*\.' + '|.*\.'.join(extensions)
            newlist = filesystem.filter_file_lines(dirlist, pattern)
            filesystem.makefile(output_file)
            filesystem.writelist(output_file, newlist, '\n')
            self.__counter = len(newlist)

        except (CoreSystemError, FileSystemError) as error:
            raise ReaderError(error)

    def filter_by_ignore_extension(self, target, output, extensions):
        """
        Specific filter for selected extensions

        :param str target: target list
        :param str output: output list
        :param list extensions: filtered extensions
        :return: None
        """

        try:

            target_file = self.__config.get(target)
            output_file = self.__config.get(output)
            dirlist = filesystem.read(target_file)
            dirlist = [i.strip() for i in dirlist]
            pattern = '^('
            for ext in extensions:
                pattern += '(?!\.{0})'.format(ext)
            pattern += '.)*$'
            newlist = filesystem.filter_file_lines(dirlist, pattern)
            filesystem.makefile(output_file)
            filesystem.writelist(output_file, newlist, '\n')
            self.__counter = len(newlist)
        except (CoreSystemError, FileSystemError) as error:
            raise ReaderError(error)

    def count_total_lines(self):
        """
        Count total lines inside wordlist
        :raise ReaderError
        :return: int
        """
        try:
            if 0 == self.__counter:
                if True is self.__browser_config.get('is_external_wordlist'):
                    dirlist = self.__browser_config.get('wordlist')
                else:
                    dirlist = self.__config.get(self.__browser_config.get('list'))
                self.__counter = filesystem.count_lines(dirlist)

            return self.__counter

        except (TypeError, FileSystemError) as error:
            raise ReaderError(error)

    @property
    def total_lines(self):
        """
        Return total lines
        :return: int
        """

        return self.__counter



# -*- coding: utf-8 -*-

"""
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

    Development Team: Brain Storm Team
"""

import re
from src.core import filesystem
from src.core import helper


class ResponseProvider(object):
    """ ResponseProvider class"""

    HTTP_DBG_LEVEL = 3
    DEFAULT_HTTP_SUCCESS_STATUSES = [100, 101, 200, 201, 202, 203, 204, 205, 206, 207, 208]
    DEFAULT_HTTP_REDIRECT_STATUSES = [300, 301, 302, 303, 304, 307, 308]
    DEFAULT_HTTP_FAILED_STATUSES = [404, 406, 410, 409, 412, 424, 429, 440, 500, 501, 502, 503, 504, 520, 522]
    DEFAULT_SSL_CERT_REQUIRED_STATUSES = [423, 496, 525]
    DEFAULT_HTTP_FORBIDDEN_STATUSES = [403,405]
    DEFAULT_HTTP_AUTH_STATUSES = [401]
    DEFAULT_HTTP_BAD_REQUEST_STATUSES = [400, 415, 508]

    def __init__(self, config):
        """
        Response instance
        :param src.lib.browser.config.Config config: configurations
        """

        self._cfg = config
        self._response_plugins = []

    @classmethod
    def _get_redirect_url(cls, url, response):
        """
        Get redirect url
        :param str url: redirect url
        :param urllib3.response.HTTPResponse response: response object
        :return: str
        """

        redirect_url = None
        location = response.get_redirect_location()

        if location is not False:
            matches = re.search("(?P<url>https?://[^\s]+)", location)
            if matches is not None:
                redirect_url = matches.group("url")
            else:
                urlp = helper.parse_url(url)
                location = location if True is location.startswith('/') else ''.join(('/', location))
                redirect_url = urlp.scheme + '://' + urlp.netloc + location

        return redirect_url

    def detect(self, request_url, response):
        """
        Detect response by status code
        :param str request_url: request url
        :param urllib3.response.HTTPResponse response: response object
        :raise Exception
        :return: str
        """

        status = None
        for _, resp_plugin in enumerate(self._response_plugins):
            status = resp_plugin.process(response)
            if None is not status:
                break

        if None is status:
            if response.status in self.DEFAULT_HTTP_SUCCESS_STATUSES:
                return 'success'
            elif response.status in self.DEFAULT_HTTP_FAILED_STATUSES:
                return 'failed'
            elif response.status in self.DEFAULT_SSL_CERT_REQUIRED_STATUSES:
                return 'certificate'
            elif response.status in self.DEFAULT_HTTP_REDIRECT_STATUSES:
                return self.__is_redirect(response, request_url)
            elif response.status in self.DEFAULT_HTTP_BAD_REQUEST_STATUSES:
                return 'bad'
            elif response.status in self.DEFAULT_HTTP_FORBIDDEN_STATUSES:
                return 'forbidden'
            elif response.status in self.DEFAULT_HTTP_AUTH_STATUSES:
                return 'auth'
            else:
                raise Exception('Unknown response status : `{0}`'.format(response.status))
        else:
            return status

    def handle(self, response, request_url, items_size, total_size, ignore_list):
        """
        Response handler
        :param urllib3.response.HTTPResponse response: response object
        :param str request_url: url from request
        :param int items_size: current items sizes
        :param int total_size: response object
        :param list ignore_list: ignore list
        :raise ResponseError
        :return: dict
        """

        pass

    @staticmethod
    def _get_content_size(response):
        """
        Get content size
        :param urllib3.response.HTTPResponse response: response object
        :return: str
        """
        size = 0

        try:
            size = 0 if not hasattr(response, 'headers') else int(response.headers['Content-Length'])
        except (KeyError, ValueError):
            size = len(response.data)
        finally:
            size = filesystem.human_size(size, 0)
        return size

    @staticmethod
    def __is_redirect(response, request_url):
        """
        Return redirect status
        :param str request_url: request url
        :param urllib3.response.HTTPResponse response: response object
        :return: str
        """

        location = response.get_redirect_location()
        if location is not False and location is not None:
            urlfrag = helper.parse_url(request_url)
            redirect_url = location.rstrip('/')

            redirectfrag = helper.parse_url(redirect_url)
            url = "{0}://{1}".format(urlfrag.scheme, urlfrag.netloc)
            if url == redirect_url \
                    or (0 < len(redirectfrag.query) and redirectfrag.query in urlfrag.path):
                return 'failed'
            return 'redirect'
        else:
            return 'failed'



# -*- coding: utf-8 -*-

"""
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

    Development Team: Brain Storm Team
"""

from __future__ import absolute_import
import unittest
import os
import sys
import signal
from src.lib.events import EventHandler

@unittest.skipUnless(hasattr(os, 'kill'), "Test requires os.kill")
@unittest.skipIf(sys.platform =="win32", "Test cannot run on Windows")
@unittest.skipIf(sys.platform == 'freebsd6', "Test kills regrtest on freebsd6 "
    "if threads have been used")
class TestEvents(unittest.TestCase):
    """TestEvents class"""

    def test_termination_handler(self):
        """ Events.terminate() exception test """

        EventHandler.terminate()

        self.assertIs('kill_process', signal.getsignal(signal.SIGTSTP).__name__)
        self.assertTrue(None is not signal.getsignal(signal.SIGTERM))

if __name__ == "__main__":
    unittest.main()



# -*- coding: utf-8 -*-

"""
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

    Development Team: Brain Storm Team
"""

import unittest
import os
from configparser import ConfigParser

from src.core import filesystem
from src.lib.reporter import Reporter, ReporterError
from src.lib.reporter.plugins.provider import PluginProvider
from ddt import ddt, data
from src.core.logger.logger import Logger
import shutil


@ddt
class TestReporter(unittest.TestCase):
    """TestReporter class"""
    
    @property
    def __configuration(self):
        test_config =  filesystem.getabsname(os.path.join('tests', 'data', 'setup.cfg'))
        config = ConfigParser.RawConfigParser()
        config.read(test_config)
        return config
    
    def setUp(self):
        self.mockdata = {
            "items": {
                "failed": [
      "http://test.local/swfobject.js",
      "http://test.local/swfobject.php",
      "http://test.local/swfobject/",
      "http://test.local/swfs/",
      "http://test.local/swfupload.php",
      "http://test.local/swfupload/",
      "http://test.local/swift/",
      "http://test.local/login/",
      "http://test.local/users/",
      "http://test.local/swiss/",
      "http://test.local/switch.php",
      "http://test.local/switch/",
      "http://test.local/switcher/",
      "http://test.local/switchover/",
      "http://test.local/switchoverfiles/"
    ],
                "success": [
      "http://test.local/cron/exp.php",
      "http://test.local/cron/cron-online.php"
    ]
            },
            "total": {
    "failed": 15,
    "items": 17,
    "success": 2,
    "workers": 1
            }
        }
        PluginProvider.CONFIG_FILE = 'tests/data/setup.cfg'
        
    def tearDown(self):
        logger = Logger.log()
        for handler in logger.handlers:
            logger.removeHandler(handler)
        
    def test_is_reported(self):
        """ Reporter.is_reported() test """

        expected = Reporter.is_reported('resource')
        self.assertIs(type(expected), bool)
        
    # def test_is_reported_error(self):
    #     """ Reporter.is_reported() exception test """
    #     rp = Reporter
    #     setattr(rp, 'config', 'wrong.cfg')
    #     with self.assertRaises(ReporterError) as context:
    #         rp.is_reported('resource')
    #
    #         self.assertTrue(ReporterError == context.expected)
        
    def test_plugin_provider_invalid_data_exception(self):
        """ PluginProvider.init() exception test """

        with self.assertRaises(TypeError) as context:
            PluginProvider('test.local', 'wrongdata')
            self.assertTrue(TypeError == context.expected)

    def test_plugin_provider_invalid_record(self):
        """ PluginProvider.record() exception test """

        provider = PluginProvider('test.local', self.mockdata)
        with self.assertRaises(Exception) as context:
            provider.record('wrongdir', self.mockdata)
            self.assertTrue(Exception == context.expected)

    @data('std', 'txt', 'json', 'html')
    def test_load(self, value):
        """ Reporter.load() test """

        expected = Reporter.load(value, 'test.local', {})
        self.assertIsInstance(expected, PluginProvider)
        
    # @data('std', 'txt', 'json', 'html')
    # def test_process(self, ext):
    #     """ Reporter.load().process() test """
    #
    #     report = Reporter.load(ext, 'test.local', self.mockdata)
    #     self.assertIsNone(report.process())
    #     if ext in ['html','json']:
    #         self.assertTrue(filesystem.is_exist('tests/reports/test.local', 'test.local.{0}'.format(ext)))
    #     if ext in ['txt']:
    #         self.assertTrue(filesystem.is_exist('tests/reports/test.local', 'success.{0}'.format(ext)))
    #     shutil.rmtree('tests/reports')
        
    # @data('std', 'txt', 'json', 'html')
    # def test_load_plugin_exception(self, ext):
    #     """ Reporter.load() exception test """
    #
    #     if ext in ['html','json', 'txt']:
    #
    #         PluginProvider.CONFIG_FILE = 'wrong.cfg'
    #         with self.assertRaises(Exception) as context:
    #             Reporter.load(ext, 'test.local', self.mockdata)
    #             self.assertTrue(Exception == context.expected)
       

    def test_load_exception(self):
        """ Reporter.load() exception test """

        undefined = 'undefined'
        with self.assertRaises(ReporterError) as context:
            Reporter.load(undefined,'test.local', {})
            self.assertTrue('Unable to get reporter `{plugin}`'.format(plugin=undefined) in context.exception)
            self.assertTrue(ReporterError == context.expected)

if __name__ == "__main__":
    unittest.main()



# -*- coding: utf-8 -*-

"""
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

    Development Team: Brain Storm Team
"""

import sys
# noinspection PyCompatibility
from argparse import RawDescriptionHelpFormatter, OPTIONAL
from .exceptions import ArgumentParserError, ThrowingArgumentParser, OptionsError, FilterError
from .filter import Filter


class Options(object):

    """Options class"""

    def __init__(self):
        """
        Constructor
        :raise OptionsError
        """

        self.__standalone = ['version', 'update', 'examples', 'docs']
        self.__wizard_conf = 'opendoor.conf'

        __groups = {
            'request': "Request tools",
            'stream': "Stream tools",
            'debug': "Debug tools",
            'wordlist': "Wordlist tools",
            'sniff': "Sniff tools",
            'report': "Reports tools",
            'app': "Application tools"
        }

        __arguments = [
            {
                "group": "request",
                "args": "-p",
                "argl": "--port",
                "default": 80,
                "action": "store",
                "help": "Custom port (Default 80)",
                "type": int
            },
            {
                "group": "request",
                "args": "-m",
                "argl": "--method",
                "default": "HEAD",
                "action": "store",
                "help": "Request method (use HEAD as default)",
                "type": str
            },
            {
                "group": "stream",
                "args": "-t",
                "argl": "--threads",
                "default": 1,
                "action": "store",
                "help": "Allowed threads",
                "type": int
            },
            {
                "group": "request",
                "args": "-d",
                "argl": "--delay",
                "default": 0,
                "action": "store",
                "help": "Delay between requests threading",
                "type": float
            },
            {
                "group": "request",
                "args": None,
                "argl": "--timeout",
                "default": 30,
                "action": "store",
                "help": "Request timeout (30 sec default)",
                "type": int
            },
            {
                "group": "request",
                "args": "-r",
                "argl": "--retries",
                "default": 3,
                "action": "store",
                "help": "Max retries to reconnect (default 3)",
                "type": int
            },
            {
                "group": "request",
                "args": None,
                "argl": "--keep-alive",
                "default": False,
                "action": "store_true",
                "help": "Use keep-alive connection",
                "type": bool
            },
            {
                "group": "request",
                "args": None,
                "argl": "--accept-cookies",
                "default": False,
                "action": "store_true",
                "help": "Accept and route cookies from responses",
                "type": bool
            },
            {
                "group": "debug",
                "args": None,
                "argl": "--debug",
                "default": 0,
                "action": "store",
                "help": "Debug level -1 (silent), 1 - 3",
                "type": int
            },
            {
                "group": "request",
                "args": None,
                "argl": "--tor",
                "default": False,
                "action": "store_true",
                "help": "Using built-in proxylist",
                "type": bool
            },
            {
                "group": "request",
                "args": None,
                "argl": "--torlist",
                "default": None,
                "action": "store",
                "help": "Path to custom proxylist",
                "type": str
            },
            {
                "group": "request",
                "args": None,
                "argl": "--proxy",
                "default": None,
                "action": "store",
                "help": "Custom permanent proxy server",
                "type": str
            },
            {
                "group": "wordlist",
                "args": "-s",
                "argl": "--scan",
                "default": "directories",
                "action": "store",
                "help": "Scan type scan=directories or scan=subdomains",
                "type": str
            },
            {
                "group": "wordlist",
                "args": "-w",
                "argl": "--wordlist",
                "default": None,
                "action": "store",
                "help": "Path to custom wordlist",
                "type": str
            },
            {
                "group": "report",
                "args": None,
                "argl": "--reports",
                "default": "std",
                "action": "store",
                "help": "Scan reports (json,std,txt,html)",
                "type": str
            },
            {
                "group": "report",
                "args": None,
                "argl": "--reports-dir",
                "default": None,
                "action": "store",
                "help": "Path to custom reports dir",
                "nargs": 1,
                "type": str
            },
            {
                "group": "request",
                "args": None,
                "argl": "--random-agent",
                "default": False,
                "action": "store_true",
                "help": "Randomize user-agent per request",
                "type": bool
            },
            {
                "group": "wordlist",
                "args": None,
                "argl": "--random-list",
                "default": False,
                "action": "store_true",
                "help": "Shuffle scan list",
                "type": bool
            },
            {
                "group": "wordlist",
                "args": None,
                "argl": "--prefix",
                "default": None,
                "action": "store",
                "help": "Append path prefix to scan host",
                "type": str
            },
            {
                "group": "wordlist",
                "args": "-e",
                "argl": "--extensions",
                "default": None,
                "action": "store",
                "help": "Force use selected extensions for scan session -e php,json e.g",
                "type": str
            },
            {
                "group": "wordlist",
                "args": "-i",
                "argl": "--ignore-extensions",
                "default": None,
                "action": "store",
                "help": "Force ignore extensions for scan session -i aspx,jsp e.g",
                "type": str
            },
            {
                "group": "sniff",
                "args": None,
                "argl": "--sniff",
                "default": None,
                "action": "store",
                "help": "Response sniff plugins (indexof,collation,file,skipempty)",
                "type": str
            },
            {
                "group": "app",
                "args": None,
                "argl": "--update",
                "default": False,
                "action": "store_true",
                "help": "Update from CVS",
                "type": bool
            },
            {
                "group": "app",
                "args": None,
                "argl": "--version",
                "default": False,
                "action": "store_true",
                "help": "Get current version",
                "type": bool
            },
            {
                "group": "app",
                "args": None,
                "argl": "--examples",
                "default": False,
                "action": "store_true",
                "help": "Examples of usage",
                "type": bool
            },
            {
                "group": "app",
                "args": None,
                "argl": "--docs",
                "default": False,
                "action": "store_true",
                "help": "Read documentation",
                "type": bool
            },
            {
                "group": "app",
                "args": None,
                "argl": "--wizard",
                "default": None,
                "action": "store",
                "help": "Run wizard scanner from your config",
                "nargs": OPTIONAL,
                "const": self.__wizard_conf,
                "type": str
            }
        ]

        groupped = {}
        try:
            self.parser = ThrowingArgumentParser(formatter_class=RawDescriptionHelpFormatter)
            required_named = self.parser.add_argument_group('required named options')
            required_named.add_argument('--host', help="Target host (ip); --host http://example.com")
            arguments_len = len(__arguments)

            for group, description in sorted(__groups.items()):
                groupped[group] = self.parser.add_argument_group(description)

            for i in range(arguments_len):
                arg = __arguments[i]

                if 'nargs' in arg and arg['nargs'] == '?':
                    const = arg['const']
                else:
                    const = None

                if arg['args'] is None:
                    if bool == arg['type']:
                        groupped[arg['group']].add_argument(arg['argl'],
                                                            default=arg['default'],
                                                            action=arg['action'],
                                                            help=arg['help'])
                    elif None is not const:
                        groupped[arg['group']].add_argument(arg['argl'],
                                                            default=arg['default'],
                                                            action=arg['action'],
                                                            help=arg['help'],
                                                            nargs=arg['nargs'],
                                                            const=const,
                                                            type=arg['type'])
                    else:
                        groupped[arg['group']].add_argument(arg['argl'],
                                                            default=arg['default'],
                                                            action=arg['action'],
                                                            help=arg['help'],
                                                            type=arg['type'])
                else:
                    if bool == arg['type']:
                        groupped[arg['group']].add_argument(arg['args'],
                                                            arg['argl'],
                                                            default=arg['default'],
                                                            action=arg['action'],
                                                            help=arg['help'])
                    elif None is not const:
                        groupped[arg['group']].add_argument(arg['args'],
                                                            arg['argl'],
                                                            default=arg['default'],
                                                            action=arg['action'],
                                                            help=arg['help'],
                                                            nargs=arg['nargs'],
                                                            const=const,
                                                            type=arg['type'])
                    else:
                        groupped[arg['group']].add_argument(arg['args'],
                                                            arg['argl'],
                                                            default=arg['default'],
                                                            action=arg['action'],
                                                            help=arg['help'],
                                                            type=arg['type'])

            self.args = self.parser.parse_args()
        except (ArgumentParserError, KeyError) as error:
            raise OptionsError(error)

    def get_arg_values(self):
        """
        Get used input options
        :raise OptionsError
        :return: dict
        """

        args = {}

        try:
            arguments = self.args

            if not self.args.host \
                    and True is not self.args.version \
                    and True is not self.args.update \
                    and True is not self.args.docs \
                    and True is not self.args.examples \
                    and None is self.args.wizard:
                sys.exit(self.parser.print_help())

            if True is self.args.version or True is self.args.update \
                    or True is self.args.examples or True is self.args.docs:
                for arg, value in vars(arguments).items():
                    if arg in self.__standalone and True is value:
                        args[arg] = value
                        break
            else:

                for arg, value in vars(self.args).items():

                    if value:
                        args[arg] = value
                args = Filter.filter(args)

            return args

        except (AttributeError, FilterError, ArgumentParserError) as error:
            raise OptionsError(error)



# -*- coding: utf-8 -*-

"""
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

    Development Team: Brain Storm Team
"""

from __future__ import absolute_import
import unittest
from src.core.system.terminal import Terminal
from src.core.system.output import Output

@unittest.skipIf(False is Output().is_windows, "Test can run on Windows")
class TestTerminal(unittest.TestCase):
    """TestTerminal class"""
    
    # def test_get_ts_windows(self):
    #     """ Terminal.__get_ts_windows() test """
    #
    #     term = getattr(Terminal, '_Terminal__get_ts_windows')()
    #     self.assertIsNotNone(term)

        
if __name__ == "__main__":
    unittest.main()



# -*- coding: utf-8 -*-

"""
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

    Development Team: Brain Storm Team
"""

import importlib
from .exceptions import ResponsePluginError


class ResponsePlugin(object):
    """ResponsePlugin class"""

    @staticmethod
    def load(plugin):
        """
        Load response plugin
        :param str plugin:  response plugin name
        :raise ResponsePluginError
        :return: src.core.http.plugins.response.provider.provider.ResponsePluginProvider
        """

        plugin_value = None

        try:
            plugin_name, plugin_value = plugin.split('=')
        except ValueError:
            plugin_name = plugin
            pass
        try:
            package_module = importlib.import_module('src.core.http.plugins.response')

            try:
                response_plugin = getattr(package_module, plugin_name)
                return response_plugin(plugin_value)
            except (TypeError, AttributeError, Exception) as error:
                raise ResponsePluginError('Unable to get response plugin `{plugin}`. Reason: {error}'
                                          .format(plugin=plugin_name, error=error))
        except ImportError:
            raise ResponsePluginError('Unable to get response\'s plugins')



# -*- coding: utf-8 -*-

"""
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

    Development Team: Brain Storm Team
"""

class CacheControlProvider(object):
    """ CacheControlProvider class"""

    def __init__(self):
        """
        Init interface
        """

        self.__cache_control = 'max-age=0'

    @property
    def _cache_control(self):
        """
        Get 'Cache-Control' Header
        :return: str
        """

        return self.__cache_control



# -*- coding: utf-8 -*-

"""
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

    Development Team: Brain Storm Team
"""

import inspect
import logging
import logging.config

from src.core import filesystem
from .config import Config


class LoggerException(object):

    """ Exception class """

    @staticmethod
    def log(class_name='Error', message=''):
        """
        Syslog error handler
        :param str class_name: log name
        :param str message: log message
        :raise Error
        :return: None
        """

        try:

            filesystem.makedir(Config.logdir)
            logging.config.dictConfig(Config.exceptions)
            logger = logging.getLogger('exceptions')
            func = inspect.currentframe().f_back.f_code
            message = "{class_name}: {message} in {file} -> {func}() line {line}".format(
                    class_name=class_name,
                    message=message,
                    file=func.co_filename,
                    func=func.co_name,
                    line=func.co_firstlineno)
            logger.error(message)
        except (Exception, ValueError):
            # skip error undefined owner
            pass



# -*- coding: utf-8 -*-

"""
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

    Development Team: Brain Storm Team
"""

import unittest
from src.lib.browser.worker import Worker
# noinspection PyCompatibility
from queue import Queue


class TestBrowserWorker(unittest.TestCase):
    """TestBrowserWorker class"""
    
    THREADS = 1
    
    def _kill_waiting_thread(self):
        exit()
        
    def setUp(self):
        self._worker = Worker(Queue(self.THREADS), num_threads=self.THREADS, timeout=0)
    
    def tearDown(self):
        with self.assertRaises(SystemExit):
            self._kill_waiting_thread()
            self._worker.join(1)

    def test_pause(self):
        """ Worker.pause() test """
        
        self.assertEqual(self._worker.pause(), None)
        self.assertFalse(getattr(self._worker, '_Worker__running', True))

if __name__ == "__main__":
    unittest.main()


# -*- coding: utf-8 -*-

"""
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

    Development Team: Brain Storm Team
"""

# noinspection PyPep8Naming
from .color import Color as colour
# noinspection PyPep8Naming
from .core import CoreConfig
# noinspection PyPep8Naming
from .decorators import execution_time
# noinspection PyPep8Naming
from .filesystem import FileSystem as filesystem
from .filesystem.exceptions import FileSystemError
# noinspection PyPep8Naming
from .helper import Helper as helper
# noinspection PyPep8Naming
from .http import HttpRequest as request_http
# noinspection PyPep8Naming
from .http import HttpsRequest as request_https
# noinspection PyPep8Naming
from .http import Proxy as request_proxy
# noinspection PyPep8Naming
from .http import Response as response
# noinspection PyPep8Naming
from .http import Socket as socket
from .http.exceptions import HttpRequestError
from .http.exceptions import HttpsRequestError
from .http.exceptions import ProxyRequestError
from .http.exceptions import ResponseError
from .http.exceptions import SocketError
# noinspection PyPep8Naming
from .logger import LoggerException as exception
# noinspection PyPep8Naming
from .logger import Logger as logger
# noinspection PyPep8Naming
from .options import Options as options
from .options.exceptions import OptionsError
# noinspection PyPep8Naming
from .system import Term as process
# noinspection PyPep8Naming
from .system import Output as sys
from .system.exceptions import CoreSystemError



# -*- coding: utf-8 -*-

"""
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

    Development Team: Brain Storm Team
"""

from .provider import ResponsePluginProvider


class FileResponsePlugin(ResponsePluginProvider):
    """ FileResponsePlugin class"""

    DESCRIPTION = 'File (detect large files)'
    RESPONSE_INDEX = 'file'
    DEFAULT_STATUSES = [100, 101, 200, 201, 202, 203, 204, 205, 206, 207, 208]
    DEFAULT_SOURCE_DETECT_MIN_SIZE = 1000000

    def __init__(self, void):
        """
        ResponsePluginProvider constructor
        """

        ResponsePluginProvider.__init__(self)

    def process(self, response):
        """
        Process data
        :return: str
        """

        if response.status in self.DEFAULT_STATUSES:
            super().process(response)
            if 'Content-Length' in self._headers:
                if self.DEFAULT_SOURCE_DETECT_MIN_SIZE <= int(self._headers['Content-Length']):
                    return self.RESPONSE_INDEX
                elif self.DEFAULT_SOURCE_DETECT_MIN_SIZE <= len(self._body):
                    return self.RESPONSE_INDEX
        return None



# -*- coding: utf-8 -*-

"""
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

    Development Team: Brain Storm Team
"""

from urllib3 import HTTPSConnectionPool, PoolManager, HTTPResponse, Timeout, disable_warnings
from urllib3.exceptions import MaxRetryError, ReadTimeoutError, ConnectTimeoutError, \
    HostChangedError, SSLError, InsecureRequestWarning
from src.core import helper
from .exceptions import HttpsRequestError
from .providers import DebugProvider
from .providers import RequestProvider


class HttpsRequest(RequestProvider, DebugProvider):

    """HttpsRequest class"""

    DEFAULT_SSL_CERT_REQUIRED_STATUSES = 496

    def __init__(self, config, debug, **kwargs):
        """
        HttpsRequest instance
        :param src.lib.browser.config.Config config: global configurations
        :param DebugProvider debug: debugger
        """

        try:
            self.__tpl = kwargs.get('tpl')
            RequestProvider.__init__(self, config, agent_list=kwargs.get('agent_list'))
            self.__headers = self._headers
            self.__connection_header = 'default'
            if True is config.keep_alive:
                self.__connection_header = self._keep_alive
        except (TypeError, ValueError) as error:
            raise HttpsRequestError(error)

        self.__cfg = config
        self.__debug = debug
        if self.__cfg.DEFAULT_SCAN == self.__cfg.scan:
            self.__pool = self.__https_pool()
    
    def _provide_ssl_auth_required(self):
        """
        Provide ssl auth response
        :return: urllib3.HTTPResponse
        """
        
        response = HTTPResponse()
        response.status = self.DEFAULT_SSL_CERT_REQUIRED_STATUSES
        response.__setattr__('_body', ' ')
        return response
        
    def __https_pool(self):
        """
        Create HTTP connection pool
        :raise HttpsRequestError
        :return: urllib3.HTTPConnectionPool
        """

        try:
            pool = HTTPSConnectionPool(
                    host=self.__cfg.host,
                    port=self.__cfg.port,
                    maxsize=self.__cfg.threads,
                    timeout=Timeout(connect=self.__cfg.timeout, read=self.__cfg.timeout),
                    cert_reqs='CERT_NONE',
                    )
            if self._HTTP_DBG_LEVEL <= self.__debug.level:
                self.__debug.debug_connection_pool('https_pool_start', pool, self.__connection_header)

            return pool
        except Exception as error:
            raise HttpsRequestError(str(error))

    def request(self, url):
        """
        Client request SSL
        :param str url: request uri
        :return: urllib3.HTTPResponse
        """

        self.__headers.update({'User-Agent': self._user_agent})
        if 'default' is not self.__connection_header:
            self.__headers.update({'Connection': self.__connection_header})

        if self._HTTP_DBG_LEVEL <= self.__debug.level:
            self.__debug.debug_request(self.__headers, url, self.__cfg.method)
        try:
            disable_warnings(InsecureRequestWarning)
            if self.__cfg.DEFAULT_SCAN == self.__cfg.scan:  # directories requests
                response = self.__pool.request(self.__cfg.method,
                                               helper.parse_url(url).path,
                                               headers=self.__headers,
                                               retries=self.__cfg.retries,
                                               assert_same_host=False,
                                               redirect=False)
                self.cookies_middleware(is_accept=self.__cfg.accept_cookies, response=response)
            else:  # subdomains

                response = PoolManager().request(self.__cfg.method, url,
                                                 headers=self.__headers,
                                                 retries=self.__cfg.retries,
                                                 assert_same_host=False,
                                                 redirect=False,
                                                 timeout=Timeout(connect=self.__cfg.timeout, read=self.__cfg.timeout))
            return response

        except MaxRetryError:
            if self.__cfg.DEFAULT_SCAN == self.__cfg.scan:
                self.__tpl.warning(key='max_retry_error', url=helper.parse_url(url).path)

        except HostChangedError as error:
            self.__tpl.warning(key='host_changed_error', details=error)
            pass

        except ReadTimeoutError:
            self.__tpl.warning(key='read_timeout_error', url=url)
            pass

        except ConnectTimeoutError:
            self.__tpl.warning(key='connection_timeout_error', url=url)
            pass

        except SSLError:
            if self.__cfg.DEFAULT_SCAN != self.__cfg.scan:
                return self._provide_ssl_auth_required()



# -*- coding: utf-8 -*-

"""
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

    Development Team: Brain Storm Team
"""


class SocketError(Exception):

    """SocketError class"""

    def __init__(self, message):
        """
        SocketError class constructor
        :param str message: error message
        """

        super(SocketError, self).__init__(message)


class ProxyRequestError(Exception):
    """ProxyRequestError class"""

    def __init__(self, message):
        """
        ProxyRequestError class constructor
        :param str message: error message
        """

        super(ProxyRequestError, self).__init__(message)


class HttpRequestError(Exception):
    """HttpRequestError class"""

    def __init__(self, message):
        """
        HttpRequestError class constructor
        :param str message: error message
        """

        super(HttpRequestError, self).__init__(message)


class HttpsRequestError(Exception):
    """HttpsRequestError class"""

    def __init__(self, message):
        """
        HttpsRequestError class constructor
        :param str message: error message
        """

        super(HttpsRequestError, self).__init__(message)


class ResponseError(Exception):
    """ResponseError class"""

    def __init__(self, message):
        """
        ResponseError class constructor
        :param str message: error message
        """

        super(ResponseError, self).__init__(message)



# -*- coding: utf-8 -*-

"""
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

    Development Team: Brain Storm Team
"""

import importlib
import random

from urllib3 import ProxyManager, Timeout, disable_warnings
from urllib3.exceptions import DependencyWarning, MaxRetryError, ProxySchemeUnknown, \
    ReadTimeoutError, InsecureRequestWarning

from src.core import helper
from .exceptions import ProxyRequestError
from .providers import DebugProvider
from .providers import RequestProvider


class Proxy(RequestProvider, DebugProvider):
    """Proxy class"""

    def __init__(self, config, debug, **kwargs):
        """
        Proxy instance
        :param src.lib.browser.config.Config config: global configurations
        :param DebugProvider debug: debugger
        """

        try:
            self.__tpl = kwargs.get('tpl')
            self.__proxylist = kwargs.get('proxy_list')
            RequestProvider.__init__(self, config, agent_list=kwargs.get('agent_list'))
            self.__headers = self._headers
            self.__connection_header = 'default'
            if True is config.keep_alive:
                self.__connection_header = self._keep_alive
            self.__server = None
            self.__pm = None
            self.__cfg = config
            self.__debug = debug
            self.__debug.debug_proxy_pool()

            if False is config.is_standalone_proxy and 0 == len(self.__proxylist):
                raise TypeError('Proxy list empty or has invalid format')

        except (TypeError, ValueError) as error:
            raise ProxyRequestError(error)

    def __proxy_pool(self):
        """
        Create Proxy connection pool
        :raise ProxyRequestError
        :return: urllib3.HTTPConnectionPool
        """

        try:

            self.__server = self.__cfg.proxy if True is self.__cfg.is_standalone_proxy else self.__get_random_proxy()

            if self.__get_proxy_type(self.__server) == 'socks':

                disable_warnings(InsecureRequestWarning)

                if not hasattr(self, '__pm'):
                    package_module = importlib.import_module('urllib3.contrib.socks')
                    self.__pm = getattr(package_module, 'SOCKSProxyManager')

                pool = self.__pm(self.__server,
                                 num_pools=self.__cfg.threads,
                                 timeout=Timeout(self.__cfg.timeout,
                                                 read=self.__cfg.timeout),
                                 block=True)
            else:
                pool = ProxyManager(self.__server,
                                    num_pools=self.__cfg.threads,
                                    timeout=Timeout(connect=self.__cfg.timeout, read=self.__cfg.timeout),
                                    block=True)
            return pool
        except (DependencyWarning, ProxySchemeUnknown, ImportError) as error:
            raise ProxyRequestError(error)

    def request(self, url):
        """
        Client request using Proxy
        :param str url: request uri
        :return: urllib3.HTTPResponse
        """

        self.__headers.update({'User-Agent': self._user_agent})
        if 'default' is not self.__connection_header:
            self.__headers.update({'Connection': self.__connection_header})

        if self._HTTP_DBG_LEVEL <= self.__debug.level:
            self.__debug.debug_request(self.__headers, url, self.__cfg.method)

        try:
            response = self.__pool_request(url)
            return response

        except MaxRetryError:
            if self.__cfg.DEFAULT_SCAN == self.__cfg.scan:
                self.__tpl.warning(key='proxy_max_retry_error', url=helper.parse_url(url).path, proxy=self.__server)
                # Retrying request
                return self.__pool_request(url)

        except ReadTimeoutError:
            if self.__cfg.DEFAULT_SCAN == self.__cfg.scan:
                self.__tpl.warning(key='read_timeout_error', url=helper.parse_url(url).path)

    def __pool_request(self, url):
        """
        Shadow pool request
        :param string url: target url
        :return: urllib3.HTTPResponse
        """

        pool = self.__proxy_pool()
        response = pool.request(self.__cfg.method, url, headers=self.__headers, retries=self.__cfg.retries,
                                redirect=False)

        self.cookies_middleware(is_accept=self.__cfg.accept_cookies, response=response)

        return response

    def __get_random_proxy(self):
        """
        Get random server from proxy list
        :return: str
        """

        index = random.randrange(0, len(self.__proxylist))
        server = self.__proxylist[index].strip()
        return server

    @classmethod
    def __get_proxy_type(cls, server):
        """
        Set proxy type
        :param str server: input proxy server
        :return: str
        """

        if 'socks' in server:
            return 'socks'
        elif 'https' in server:
            return 'https'
        else:
            return 'http'



# -*- coding: utf-8 -*-

"""
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

    Development Team: Brain Storm Team
"""

import ctypes
import logging
import os
import re


class ColorizingStreamHandler(logging.StreamHandler):

    """ ColorizingStreamHandler class"""

    # color names to indices
    color_map = {
        'black': 0,
        'red': 1,
        'green': 2,
        'yellow': 3,
        'blue': 4,
        'magenta': 5,
        'cyan': 6,
        'white': 7
    }

    # levels to (background, foreground, bold/intense)
    level_map = {
        logging.DEBUG: (None, 'blue', False),
        logging.INFO: (None, 'black', False),
        logging.WARNING: (None, 'yellow', False),
        logging.ERROR: (None, 'red', False),
        logging.CRITICAL: ('red', 'white', True)
    }

    csi = '\x1b['
    reset = '\x1b[0m'

    @property
    def is_tty(self):
        """
        Is tty output check
        :return: bool
        """

        isatty = getattr(self.stream, 'isatty', None)
        return isatty and isatty()

    def emit(self, record):
        """
        Emmit message
        :param str record: message
        :raise Error
        :return: None
        """

        try:
            message = self.format(record)
            stream = self.stream
            if not self.is_tty:
                stream.write(message)
            else:
                self.output_colorized(message)
            stream.write(getattr(self, 'terminator', '\n'))
            self.flush()
        except (KeyboardInterrupt, SystemExit):
            raise
        except Exception as error:
            record.msg = error
            self.handleError(record)

    if os.name != 'nt':
        def output_colorized(self, message):
            """
            Prepare colorized string
            :param str message: message
            :return: None
            """

            self.stream.write(message)

    else:
        ansi_esc = re.compile(r'\x1b\[((?:\d+)(?:;(?:\d+))*)m')

        nt_color_map = {
                            0: 0x00,  # black
                            1: 0x04,  # red
                            2: 0x02,  # green
                            3: 0x06,  # yellow
                            4: 0x01,  # blue
                            5: 0x05,  # magenta
                            6: 0x03,  # cyan
                            7: 0x07  # white
                        }

        def output_colorized(self, message):
            """
            Prepare colorized string
            :param str message: message
            :return: None
            """

            parts = self.ansi_esc.split(message)
            write = self.stream.write
            h = None
            fd = getattr(self.stream, 'fileno', None)
            if fd is not None:
                fd = fd()
                if fd in (1, 2):  # stdout or stderr
                    h = ctypes.windll.kernel32.GetStdHandle(-10 - fd)
            while parts:
                text = parts.pop(0)
                if text:
                    write(text)
                if parts:
                    params = parts.pop(0)
                    if h is not None:
                        params = [int(p) for p in params.split(';')]
                        color = 0
                        for p in params:
                            if 40 <= p <= 47:
                                color |= self.nt_color_map[p - 40] << 4
                            elif 30 <= p <= 37:
                                color |= self.nt_color_map[p - 30]
                            elif p == 1:
                                color |= 0x08  # foreground intensity on
                            elif p == 0:  # reset to default color
                                color = 0x07
                            else:
                                pass  # error condition ignored
                        ctypes.windll.kernel32.SetConsoleTextAttribute(h, color)



# -*- coding: utf-8 -*-

"""
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

    Development Team: Brain Storm Team
"""

from src.core import options, OptionsError
from src.core import helper
from .exceptions import ArgumentsError


class Arguments(object):

    """Arguments class"""
    
    @staticmethod
    def get_arguments():
        """
        Get input arguments with their options
        :raise ArgumentsError
        :return: dict
        """

        try:
            args = options().get_arg_values()
            return args
        except OptionsError as error:
            raise ArgumentsError(str(error))

    @staticmethod
    def is_arg_callable(arg):
        """
        Check if argument is callable

        :param callable arg:
        :return: bool
        """

        return helper.is_callable(arg)



# -*- coding: utf-8 -*-

"""
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

    Development Team: Brain Storm Team
"""

import importlib
import os
from src.core import CoreConfig
from src.core import filesystem, FileSystemError
from .exceptions import ReporterError


class Reporter(object):

    """Reporter class"""

    default = 'std'
    external_directory = None
    config = CoreConfig.get('data')

    @staticmethod
    def is_reported(resource):
        """
        Check if session is already reported
        :param str resource: target report
        :return: bool
        """

        try:
            if None is not Reporter.external_directory:
                if not Reporter.external_directory.endswith(os.path.sep):
                    Reporter.external_directory += os.path.sep
                is_reported = filesystem.is_exist(Reporter.external_directory, resource)
            else:
                is_reported = filesystem.is_exist(Reporter.config.get('reports'), resource)
            return is_reported
        except FileSystemError as error:
            raise ReporterError(error)

    @staticmethod
    def load(plugin_name, target, data):
        """
        Load report plugin
        :param str plugin_name:  name
        :param str target: target host
        :param dict data: report data
        :raise ReporterError
        :return: src.lib.reporter.plugins.provider.provider.PluginProvider
        """

        try:
            package_module = importlib.import_module('src.lib.reporter.plugins')

            try:
                report = getattr(package_module, plugin_name)
                return report(target, data, Reporter.external_directory)
            except (TypeError, AttributeError, Exception) as error:
                raise ReporterError('Unable to get reporter `{plugin}`. Reason: {error}'
                                    .format(plugin=plugin_name, error=error))
        except ImportError:
            raise ReporterError('Unable to get report\'s plugins`')



# -*- coding: utf-8 -*-

"""
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

    Development Team: Brain Storm Team
"""

import os
import signal
import subprocess
from .exceptions import CoreSystemError
from .terminal import Terminal


class Process(object):

    """ Process class"""

    def __init__(self, classname=None, typeobj=None, params=None):
        """
        Init metaclass
        :param classname: passed classname
        :param str typeobj: passed classname type
        :param dict params: passed params
        """

        del classname, typeobj, params
        self.ts = None

    @property
    def terminal_size(self):
        """
        Get terminal window size
        :raise CoreSystemError
        :return: dict
        """

        if getattr(self, 'ts', None) is None:
            (width, height) = Terminal().get_ts()
            self.ts = {'height': height, 'width': width}
        return self.ts

    @staticmethod
    def termination_handler():
        """
        Exit Ctrl-Z handler
        :return: None
        """

        def kill_process(signum, frame):
            """
            Kill process os signal
            :param int signum: signal code
            :param object frame: frame object
            :return: None
            """

            del signum
            del frame

            os.kill(os.getpid(), signal.SIGTERM)

        sig = getattr(signal, 'SIGTSTP', signal.SIGABRT)
        signal.signal(sig, kill_process)

    @staticmethod
    def kill():
        """
        Immediatelly terminate process
        :return: None
        """

        os.kill(os.getpid(), signal.SIGTERM)  # or signal.SIGKILL

    @staticmethod
    def execute(process):
        """
        Excecute OS process
        :param str process: os command
        :raise CoreSystemError
        :return: dic
        """

        try:
            pr = subprocess.Popen(process, cwd=os.getcwd(), shell=True, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
            (out, error) = pr.communicate()
            if pr.returncode != 0:
                raise OSError(error.strip())

            return out
        except (subprocess.CalledProcessError, OSError) as error:
            raise CoreSystemError(error)


class Term(metaclass=Process):
    """Term class"""

    __metaclass__ = None
    pass



# -*- coding: utf-8 -*-

"""
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

    Development Team: Brain Storm Team
"""

from src.core.decorators import execution_time
from src.lib import ArgumentsError
from src.lib import BrowserError
from src.lib import PackageError
from src.lib import ReporterError
from src.lib import TplError
from src.lib import args
from src.lib import browser
from src.lib import events
from src.lib import package
from src.lib import reporter
from src.lib import tpl
from .exceptions import SrcError


class Controller(object):

    """Controller class"""

    def __init__(self):
        """
        Init constructor
        :raise SrcError
        """

        events.terminate()

        try:

            interpreter = package.check_interpreter()

            if interpreter is not True:
                raise SrcError(tpl.error(key='unsupported', actual=interpreter.get('actual'),
                                         expected=interpreter.get('expected')))
            else:
                self.ioargs = args().get_arguments()
        except ArgumentsError as error:
            raise SrcError(tpl.error(error))

    @execution_time(log=tpl)
    def run(self):
        """
        Bootstrap action
        :raise SrcError
        :return: None
        """

        try:

            tpl.message(package.banner())
            if 'host' in self.ioargs or 'wizard' in self.ioargs:
                getattr(self, 'scan_action')(self.ioargs)
            else:
                for action in self.ioargs.keys():

                    if hasattr(self, '{0}_action'.format(action))\
                            and args().is_arg_callable(getattr(self, '{0}_action'.format(action))):
                        getattr(self, '{func}_action'.format(func=action))()
                        break

        except (SrcError, PackageError, BrowserError, AttributeError) as error:
            raise SrcError(tpl.error(error))

    @staticmethod
    def examples_action():
        """
        Show examples action
        :return: None
        """

        tpl.message(package.examples())

    @staticmethod
    def update_action():
        """
        App update action
        :raise SrcError
        :return: None
        """

        try:
            tpl.message(package.update())
        except (AttributeError, PackageError) as error:
            raise SrcError(error)

    @staticmethod
    def docs_action():
        """
        Displays the user guide for the app.

        :raises SrcError: If there is an error with the package or attribute.
        :return: None
        """
        try:
            package.docs()
        except (AttributeError, PackageError) as error:
            raise SrcError(error)

    @staticmethod
    def version_action():
        """
        Show app version action

        :raise: SrcError
        :return: None
        """

        try:
            tpl.message(package.version())
        except (AttributeError, PackageError) as error:
            raise SrcError(error)

    @staticmethod
    def local_version():
        """
        Returns the local version of the app.

        :raises SrcError: If there is an error retrieving the local version.
        :return: None
        """

        try:
            tpl.message(package.local_version())
        except (AttributeError, PackageError) as error:
            raise SrcError(error)

    @classmethod
    def scan_action(cls, params):
        """
        URL scan action
        :param dict params: console input args
        :raise SrcError
        :return: None
        """

        try:

            if 'wizard' in params:
                tpl.info(key='load_wizard', config=params['wizard'])
                params = package.wizard(params['wizard'])
            brows = browser(params)
            if True is reporter.is_reported(params.get('host')):
                try:
                    tpl.prompt(key='logged')
                except KeyboardInterrupt:
                    tpl.cancel(key='abort')

            if reporter.default is params.get('reports'):
                tpl.info(key='use_reports')

            brows.ping()
            brows.scan()
            brows.done()

        except (AttributeError, BrowserError, ReporterError, TplError) as error:
            raise SrcError(error)
        except (KeyboardInterrupt, SystemExit):
            tpl.cancel(key='abort')



# -*- coding: utf-8 -*-

"""
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

    Development Team: Brain Storm Team
"""

import unittest
import os
from configparser import RawConfigParser
from src.core.helper.helper import Helper
from src.lib.reader import Reader, ReaderError
from src.core.filesystem.filesystem import FileSystem
from src.core.system.output import Output as sys
from src.core.logger.logger import Logger


class TestReader(unittest.TestCase):
    """TestReader class"""
    
    def tearDown(self):
        logger = Logger.log()
        for handler in logger.handlers:
            logger.removeHandler(handler)

    def __uri_validator(self, urllist):
        for url in urllist:
            try:
                result = Helper.parse_url(url)
                return True if [result.scheme, result.netloc, result.path] else False
            except:
                return False

    def __callback_function(self, string):
        """ Test callback url from list """

        self.assertTrue(self.__uri_validator(string))

    @property
    def __configuration(self):
        test_config =  FileSystem.getabsname(os.path.join('tests', 'data', 'setup.cfg'))
        config = RawConfigParser()
        config.read(test_config)
        return config

    @property
    def __configuration_for_exception(self):
        test_config =  FileSystem.getabsname(os.path.join('tests', 'data', 'setupwrong.cfg'))
        config = RawConfigParser()
        config.read(test_config)
        return config

    def test_get_user_agents_exception(self):
        """ Reader.get_user_agents() exception test """

        reader = Reader(browser_config={})
        setattr(reader, '_Reader__config', self.__configuration_for_exception)
        with self.assertRaises(ReaderError) as context:
            reader.get_user_agents()
            self.assertTrue(ReaderError == context.expected)

    def test_get_proxies_exception(self):
        """ Reader.get_proxies() exception test """

        reader = Reader(browser_config={
            'is_standalone_proxy' : False,
            'is_external_torlist' : True,
            'torlist' : '/notfound'
        })
        setattr(reader, '_Reader__config', self.__configuration_for_exception)
        with self.assertRaises(ReaderError) as context:
            reader.get_proxies()
            self.assertTrue(ReaderError == context.expected)

    def test_get_ignored_list_exception(self):
        """ Reader.get_ignored_list() exception test """

        reader = Reader(browser_config={})
        setattr(reader, '_Reader__config', self.__configuration_for_exception)
        with self.assertRaises(ReaderError) as context:
            reader.get_ignored_list()
            self.assertTrue(ReaderError == context.expected)

    def test_get_user_agents_empty(self):
        """ Reader.get_user_agents() empty test """

        empty_reader = Reader(browser_config={})
        self.assertIs(type(empty_reader.get_user_agents()), list)

    def test_get_ignored_list_empty(self):
        """ Reader.get_ignored_list() empty test """

        empty_reader = Reader(browser_config={})
        self.assertIs(type(empty_reader.get_ignored_list()), list)

    # def test_get_proxies(self):
    #     """ Reader.get_proxies() test """
    #
    #     reader = Reader(browser_config={
    #         'is_standalone_proxy' : False
    #     })
    #     setattr(reader, '_Reader__config', self.__configuration)
    #     proxies = reader.get_proxies()
    #     self.assertIs(type(proxies), list)
    #     self.assertTrue(0 < len(proxies))

    def test_get_proxies_empty(self):
        """ Reader.get_proxies() empty test """

        empty_reader = Reader(browser_config={})
        self.assertIs(type(empty_reader.get_proxies()), list)

    def test_get_lines_exception(self):
        """ Reader.get_lines() exception test """

        reader = Reader(browser_config={
            'is_standalone_proxy' : False,
            'list' : 'directories'
        })
        setattr(reader, '_Reader__config', self.__configuration_for_exception)
        with self.assertRaises(ReaderError) as context:
            reader.get_lines(params={}, loader=self.__callback_function)
            self.assertTrue(ReaderError == context.expected)

    # def test_get_lines_directories(self):
    #     """ Reader.get_lines() directories test """
    #
    #     reader = Reader(browser_config={
    #         'is_standalone_proxy' : False,
    #         'list' : 'directories',
    #         'prefix': '',
    #     })
    #     setattr(reader, '_Reader__config', self.__configuration)
    #     self.assertIsNone(reader.get_lines(params={
    #         'scheme': 'http://',
    #         'host': 'localhost.local',
    #         'port': 80
    #     }, loader=self.__callback_function))

    # def test_get_lines_subdomains(self):
    #     """ Reader.get_lines() subdomains test """
    #
    #     reader = Reader(browser_config={
    #         'is_standalone_proxy' : False,
    #         'list' : 'subdomains',
    #         'prefix': '',
    #     })
    #     setattr(reader, '_Reader__config', self.__configuration)
    #     self.assertIsNone(reader.get_lines(params={
    #         'scheme': 'http://',
    #         'host': 'localhost.local',
    #         'port': 80
    #     }, loader=self.__callback_function))

    def test_count_total_lines_exception(self):
        """ Reader.count_total_lines() exception test """

        reader = Reader(browser_config={
            'is_standalone_proxy' : False,
            'list' : 'subdomains',
            'prefix': '',
        })
        setattr(reader, '_Reader__config', self.__configuration_for_exception)
        with self.assertRaises(ReaderError) as context:
            reader.count_total_lines()
            self.assertTrue(ReaderError == context.expected)

    # def test_count_total_lines(self):
    #     """ Reader.count_total_lines() test """
    #
    #     reader = Reader(browser_config={
    #         'is_external_wordlist' : True,
    #         'list' : 'tests/data/directories.dat',
    #     })
    #     setattr(reader, '_Reader__config', self.__configuration)
    #     setattr(reader, '_Reader__counter', 0)
    #     self.assertIs(type(reader.count_total_lines()), int)
    #     self.assertTrue(0 < reader.count_total_lines())
    #     self.assertIs(15 ,reader.count_total_lines())

    def test_total_lines(self):
        """ Reader.total_lines test """

        empty_reader = Reader(browser_config={})
        self.assertIs(type(empty_reader.total_lines), int)

    # def test_randomize_list_exception(self):
    #     """ Reader.randomize_list exception test """
    #
    #     reader = Reader(browser_config={})
    #     setattr(reader, '_Reader__config', self.__configuration_for_exception)
    #     setattr(sys, '_Output__is_windows', False)
    #     with self.assertRaises(ReaderError) as context:
    #         reader.randomize_list('directories', 'tmplist')
    #         self.assertTrue(ReaderError == context.expected)
    #
    # @unittest.skipIf(True is sys().is_windows, "Skip test for windows")
    # def test_randomize_list_unix(self):
    #     """ Reader.randomize_list unix test """
    #
    #     reader = Reader(browser_config={
    #         'is_external_wordlist': True,
    #         'list': 'tests/data/directories.dat',
    #     })
    #     setattr(reader, '_Reader__config', self.__configuration)
    #     reader.count_total_lines()
    #     self.assertIsNone(reader.randomize_list('directories', 'tmplist'))
    #     fe = open('tests/data/directories.dat', 'r')
    #     fa = open('tests/tmp/list.tmp', 'r')
    #     expected = sum(1 for l in fe)
    #     actual = sum(1 for l in fa)
    #     self.assertIs(expected, actual)
    #
    # def test_randomize_list_windows(self):
    #     """ Reader.randomize_list windows test """
    #
    #     reader = Reader(browser_config={
    #         'is_external_wordlist': True,
    #         'list': 'tests/data/directories.dat',
    #     })
    #     setattr(reader, '_Reader__config', self.__configuration)
    #     setattr(sys, 'is_windows', True)
    #     reader.count_total_lines()
    #     self.assertIsNone(reader.randomize_list('directories', 'tmplist'))
    #     fe = open('tests/data/directories.dat', 'r')
    #     fa = open('tests/tmp/list.tmp', 'r')
    #     expected = sum(1 for l in fe)
    #     actual = sum(1 for l in fa)
    #     self.assertIs(expected, actual)

if __name__ == "__main__":
    unittest.main()


# -*- coding: utf-8 -*-

"""
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

    Development Team: Brain Storm Team
"""

from src.core import colour
from src.core import logger
from src.core import sys
from .config import Config
from .exceptions import TplError


class Tpl(object):

    """Tpl class"""

    @staticmethod
    def cancel(msg='', key='', **args):
        """
        Print message and stop propagation
        :param str msg: text message
        :param str key: message key for template
        :param mixed args: additional arguments
        :raise TplError
        :return: None
        """

        try:
            if key:
                msg = Tpl.__format_message(key, args=args)
            sys.exit(logger.log().warning(msg))
        except (AttributeError, TplError) as error:
            raise TplError(error)

    @staticmethod
    def line_log(msg='', key='', status='info', write=True, **args):
        """
        Stored colored log line to variable
        :param str msg: text message
        :param str key: message key for template
        :param str status: log status
        :param bool write: write immediatelly
        :param mixed args: additional arguments
        :raise TplError
        :return: None
        """

        try:
            if key:
                msg = Tpl.__format_message(key, args=args)
                msg = logger.inline(msg=msg, status=status)
            if True is write:
                sys.writels(msg)
            else:
                return msg
        except (AttributeError, TplError) as error:
            raise TplError(error)

    @staticmethod
    def prompt(key='', msg=None, status='info', **kwargs):
        """
        Prompt message
        :param str key: tpl message key
        :param str msg: target message
        :param str status: message status
        :param dict kwargs: additional key arguments
        :raise TplError
        :return:str
        """

        try:

            if 0 < len(key):
                msg = Tpl.line_log(key=key, status=status, write=False)
            else:
                msg = Tpl.line_log(msg, status=status, write=False, **kwargs)
            result = input(msg)
            return result

        except (AttributeError, TplError) as error:
            raise TplError(error)

    @staticmethod
    def line(msg='', key='', color='white', **args):
        """
        Stored colored line to variable
        :param str msg: text message
        :param str key: message key for template
        :param str color: color
        :param mixed args: additional arguments
        :raise TplError
        :return: str
        """

        try:
            if key:
                msg = Tpl.__format_message(key, args=args)
            return colour.colored(msg, color=color)
        except (AttributeError, TplError) as error:
            raise TplError(error)

    @staticmethod
    def message(msg, args=None, color='white'):
        """
        Simple colored message
        :param str msg: text message
        :param mixed args: additional arguments
        :param str color: color
        :return: None
        """

        if None is args:
            args = {}
        msg = colour.colored(msg.format(**args), color=color)
        sys.writeln(msg)

    @staticmethod
    def error(msg='', key='', **args):
        """
        Error log message
        :param mixed msg: text message
        :param str key: message key for template
        :param mixed args: additional arguments
        :raise TplError
        :return: None
        """

        msg = str(msg)

        try:
            if key:
                msg = Tpl.__format_message(key, args=args)
            logger.log('error').error(msg)
        except (AttributeError, TplError) as error:
            raise TplError(error)

    @staticmethod
    def warning(msg='', key='', **args):
        """
        Warning log message
        :param str msg: text message
        :param str key: message key for template
        :param mixed args: additional arguments
        :raise TplError
        :return: None
        """

        msg = str(msg)

        try:
            if key:
                msg = Tpl.__format_message(key, args=args)
            logger.log('warning').warning(msg)
        except (AttributeError, TplError) as error:
            raise TplError(error)

    @staticmethod
    def info(msg='', key='', clear=False, **args):
        """
        Info log message
        :param str msg: text message
        :param str key: message key for template
        :param mixed args: additional arguments
        :param bool clear: clear prev line
        :raise TplError
        :return: None
        """

        msg = str(msg)

        try:
            if key:
                msg = Tpl.__format_message(key, args=args)
            if True is clear:
                sys.writels("")
            logger.log('info').info(msg)
        except (AttributeError, TplError) as error:
            raise TplError(error)

    @staticmethod
    def debug(msg='', key='', **args):
        """
        Debug log message
        :param str msg: text message
        :param str key: message key for template
        :param mixed args: additional arguments
        :raise TplError
        :return: None
        """

        msg = str(msg)

        try:
            if key:
                msg = Tpl.__format_message(key, args=args)
            logger.log('debug').debug(msg)
        except (AttributeError, TplError) as error:
            raise TplError(error)

    @staticmethod
    def __format_message(key, **args):
        """
        Format message from config key
        :param str key: message
        :param mixed args: additional arguments
        :raise AttributeError
        :raise TplError
        :return: str
        """

        tpl = getattr(Config, 'templates')
        if key not in tpl:
            raise TplError('Could not find tpl option `{0}`'.format(key))
        msg = tpl[key]

        args = args.get('args')
        if not len(args):
            return msg
        else:
            return msg.format(**args)



# -*- coding: utf-8 -*-

"""
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

    Development Team: Brain Storm Team
"""

from src.core import HttpRequestError, HttpsRequestError, ProxyRequestError, ResponseError
from src.core import SocketError
from src.core import helper
from src.core import request_http
from src.core import request_proxy
from src.core import request_https
from src.core import response
from src.core import socket
from src.lib.reader import Reader, ReaderError
from src.lib.reporter import Reporter, ReporterError
# noinspection PyPep8Naming
from src.lib.tpl import Tpl as tpl
from .config import Config
from .debug import Debug
from .exceptions import BrowserError
from .filter import Filter
from .threadpool import ThreadPool


class Browser(Filter):
    """ Browser class """

    def __init__(self, params):
        """
        Browser constructor
        :param dict params: filtered input params
        :raise BrowserError
        """

        try:
            self.__client = None
            self.__config = Config(params)
            self.__debug = Debug(self.__config)
            self.__result = {'total': {}, 'items': {}}
            self.__reader = Reader(browser_config={
                'list': self.__config.scan,
                'torlist': self.__config.torlist,
                'use_random': self.__config.is_random_list,
                'use_extensions': self.__config.is_extension_filter,
                'use_ignore_extensions': self.__config.is_ignore_extension_filter,
                'is_external_wordlist': self.__config.is_external_wordlist,
                'wordlist': self.__config.wordlist,
                'is_standalone_proxy': self.__config.is_standalone_proxy,
                'is_external_torlist': self.__config.is_external_torlist,
                'prefix': self.__config.prefix
            })

            if True is self.__config.is_external_reports_dir:
                Reporter.external_directory = self.__config.reports_dir

            if self.__config.scan == self.__config.DEFAULT_SCAN:
                if True is self.__config.is_extension_filter:
                    self.__reader.filter_by_extension(target=self.__config.scan,
                                                      output='extensionlist',
                                                      extensions=self.__config.extensions
                                                      )
                elif True is self.__config.is_ignore_extension_filter:
                    self.__reader.filter_by_ignore_extension(target=self.__config.scan,
                                                             output='ignore_extensionlist',
                                                             extensions=self.__config.ignore_extensions
                                                             )
            self.__reader.count_total_lines()

            Filter.__init__(self, self.__config, self.__reader.total_lines)

            self.__pool = ThreadPool(num_threads=self.__config.threads, total_items=self.__reader.total_lines,
                                     timeout=self.__config.delay)

            self.__result = {'total': helper.counter(), 'items': helper.list()}

            self.__response = response(config=self.__config, debug=self.__debug, tpl=tpl)

        except (ResponseError, ReaderError) as error:
            raise BrowserError(error)

    def ping(self):
        """
        Check remote host for available
        :raise: BrowserError
        :return: None
        """

        try:
            tpl.info(key='checking_connect', host=self.__config.host, port=self.__config.port)
            socket.ping(self.__config.host, self.__config.port, self.__config.DEFAULT_SOCKET_TIMEOUT)
            tpl.info(key='online', host=self.__config.host, port=self.__config.port,
                     ip=socket.get_ip_address(self.__config.host))

        except SocketError as error:
            raise BrowserError(error)

    def scan(self):
        """
        Scanner
        :raise BrowserError
        :return: None
        """

        self.__debug.debug_user_agents()
        self.__debug.debug_list(total_lines=self.__pool.total_items_size)

        try:  # beginning scan processes
            if True is self.__config.is_random_list:
                if self.__config.scan == self.__config.DEFAULT_SCAN:
                    if True is self.__config.is_extension_filter:
                        setattr(self.__config, 'scan', 'extensionlist')
                    elif True is self.__config.is_ignore_extension_filter:
                        setattr(self.__config, 'scan', 'ignore_extensionlist')

                self.__reader.randomize_list(target=self.__config.scan, output='tmplist')

            tpl.info(key='scanning', host=self.__config.host)

            self.__start_request_provider()

            if True is self.__pool.is_started:
                self.__reader.get_lines(
                    params={
                        'host': self.__config.host,
                        'port': self.__config.port,
                        'scheme': self.__config.scheme
                    },
                    loader=getattr(self, '_add_urls'.format())
                )

        except (ProxyRequestError, HttpRequestError, HttpsRequestError, ReaderError) as error:
            raise BrowserError(error)

    def __start_request_provider(self):
        """
        Start selected request provider

        :return: None
        """

        if True is self.__config.is_proxy:
            self.__client = request_proxy(self.__config, proxy_list=self.__reader.get_proxies(),
                                          agent_list=self.__reader.get_user_agents(), debug=self.__debug, tpl=tpl)
        else:

            if True is self.__config.is_ssl:
                self.__client = request_https(self.__config, agent_list=self.__reader.get_user_agents(),
                                              debug=self.__debug, tpl=tpl)
            else:
                self.__client = request_http(self.__config, agent_list=self.__reader.get_user_agents(),
                                             debug=self.__debug, tpl=tpl)

    def __http_request(self, url):
        """
        Make HTTP request
        :param str url: received url
        :return: None
        """

        try:
            resp = self.__client.request(url)

            response_data = self.__response.handle(resp, request_url=url,
                                                   items_size=self.__pool.items_size,
                                                   total_size=self.__pool.total_items_size,
                                                   ignore_list=self.__reader.get_ignored_list()
                                                   )
            if None is response_data:
                self.__catch_report_data('ignored', url)
            else:
                self.__catch_report_data(response_data[0], response_data[1])

        except (HttpRequestError, HttpsRequestError, ProxyRequestError, ResponseError) as error:
            raise BrowserError(error)

    def __is_ignored(self, url):
        """
        Check if the path will be ignored
        :param str url: received url
        :return: bool
        """

        path = helper.parse_url(url).path.strip("/")
        return path in self.__reader.get_ignored_list()

    def _add_urls(self, urllist):
        """
        Add received urllist to threadpool
        :param dict urllist: read from dictionary
        :raise KeyboardInterrupt
        :return: None
        """

        try:

            for url in urllist:
                if False is self.__is_ignored(url):
                    self.__pool.add(self.__http_request, url)
                else:
                    self.__catch_report_data('ignored', url)
                    tpl.warning(
                        key='ignored_item',
                        current='{0:0{l}d}'.format(0, l=len(str(abs(self.__reader.total_lines)))),
                        total=self.__reader.total_lines,
                        item=helper.parse_url(url).path
                    )
            self.__pool.join()
        except (SystemExit, KeyboardInterrupt):
            raise KeyboardInterrupt

    def __catch_report_data(self, status, url):
        """
        Add to basket report pool
        :param str status: response status
        :param url: request url
        :return: None
        """

        self.__result['total'].update((status,))
        self.__result['items'][status] += [url]

    def done(self):
        """
        Scan finish action
        :raise BrowserError
        :return: None
        """

        self.__result['total'].update({"items": self.__pool.total_items_size})
        self.__result['total'].update({"workers": self.__pool.workers_size})

        if 0 == self.__pool.size:

            try:
                for rtype in self.__config.reports:
                    report = Reporter.load(rtype, self.__config.host, self.__result)
                    report.process()
            except ReporterError as error:
                raise BrowserError(error)
        else:
            pass



# -*- coding: utf-8 -*-

"""
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

    Development Team: Brain Storm Team
"""

import codecs
import collections
import json
import re
import webbrowser
from urllib.parse import urlparse
from packaging import version

try:
    from collections.abc import Callable
except ImportError:
    from collections import Callable


class Helper(object):

    """Helper class"""

    @staticmethod
    def counter():
        """
        Provide counter collection
        :return: collections.Counter
        """

        return collections.Counter()

    @staticmethod
    def list():
        """
        Provide dictionary collection
        :return: dict
        """

        container = collections.defaultdict(list)
        return container

    @staticmethod
    def parse_url(url):
        """
        Parse url

        :param str url: input url
        :return: dict
        """

        return urlparse(url)

    @staticmethod
    def to_json(data, sort=True, indents=4):
        """
        Json pretty print
        :param dict data: mixed data params
        :param bool sort: use sort
        :param int indents: space indents
        :return: str
        """

        json_data = json.dumps(data, sort_keys=sort, indent=indents)
        return json_data

    @staticmethod
    def to_list(string, delimiter=','):
        """
        Convert a string to a list by splitting it using the specified delimiter.

        :param string: The string to be converted to a list.
        :param delimiter: The delimiter used to split the string (default is ',').
        :return: A list containing the elements from the string.
        :rtype: list
        """

        try:
            return string.split(delimiter)
        except TypeError as error:
            raise TypeError(error)

    @staticmethod
    def openbrowser(target):
        """
        Open target path in local browser
        :param string target: url or local path
        :return: bool
        """

        return webbrowser.open(target)

    @staticmethod
    def percent(counter=0, total=1):
        """
        Return percentage
        :param int counter: current value
        :param int total: total value
        :return: str
        """

        return "{percent}%".format(percent=round(100.0 * counter / float(total), 1))

    @staticmethod
    def is_less(arg1, arg2):
        """
        Compare two numbers (< less)
        :param int arg1: right version
        :param int arg2: left version
        :return: bool
        """

        return version.parse(arg1) < version.parse(arg2)

    @staticmethod
    def is_more(arg1, arg2):
        """
        Compare two numbers (great >)
        :param int arg1: right version
        :param int arg2: left version
        :return: bool
        """

        return version.parse(arg1) > version.parse(arg2)

    @staticmethod
    def is_callable(func):
        """
        Check if function is callable

        :param callable func:
        :return: bool
        """

        return isinstance(func, Callable)

    @staticmethod
    def is_jsonable(obj):
        """
        Check if an object is json serializable

        :param callable obj:
        :return: bool
        """
        try:
            json.dumps(obj)
            return True
        except (TypeError, OverflowError):
            return False

    @staticmethod
    def decode_hostname(hostname):
        """
        Decode non-latin hostname

        param str hostname: input string
        :return: str
        """

        domain = hostname.strip().encode().decode('utf8')
        return str(domain.encode("idna").decode("utf-8"))

    @staticmethod
    def decode(str, errors='strict'):
        """
        Decode strings

        :param str str: input string
        :param str errors:error level
        :return: str
        """

        output = ''
        try:
            if len(str) < 3:
                if codecs.BOM_UTF8.startswith(str):
                    # not enough data to decide if this is a BOM
                    # => try again on the next call
                    output = ""

            elif str[:3] == codecs.BOM_UTF8:
                (output, sizes) = codecs.utf_8_decode(str[3:], errors)
            elif str[:3] == codecs.BOM_UTF16:
                output = str[3:].decode('utf16')
            else:
                # (else) no BOM present
                (output, sizes) = codecs.utf_8_decode(str, errors)
            return str(output)
        except (UnicodeDecodeError, Exception):
            # seems, its getting not a content (img, file, etc)
            try:
                return str.decode('cp1251')
            except (UnicodeDecodeError, Exception):
                return ""

    @staticmethod
    def filter_directory_string(str):
        """
        Filter directory string

        :param str string: input string
        :return: str
        """

        str = str.strip("\n")
        if True is str.startswith('/'):
            str = str[1:]

        return str.strip()

    @staticmethod
    def filter_domain_string(str):
        """
        Filter domain/subdomain string

        :param str string: input string
        :return: str
        """

        str.strip("\n")
        str = re.sub(r'[^\w\d_-]', '', str).lower()
        if not str:
            str = '_'
        return str.lower()
